# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import json
import os
import shlex
import subprocess

from cloudkittyclient.tests import utils


class BaseFunctionalTest(utils.BaseTestCase):

    def _run(self, executable, action,
             flags='', params='', fmt='-f json', stdin=None, has_output=True):
        if not has_output:
            fmt = ''
        cmd = ' '.join([executable, flags, action, params, fmt])
        cmd = shlex.split(cmd)
        p = subprocess.Popen(
            cmd, env=os.environ.copy(), shell=False,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            stdin=subprocess.PIPE if stdin else None,
        )
        stdout, stderr = p.communicate(input=stdin)
        if p.returncode != 0:
            raise RuntimeError('"{cmd}" returned {val}: {msg}'.format(
                cmd=' '.join(cmd), val=p.returncode, msg=stderr))
        return json.loads(stdout) if has_output else None

    def openstack(self, action,
                  flags='', params='', fmt='-f json',
                  stdin=None, has_output=True):
        return self._run('openstack rating', action,
                         flags, params, fmt, stdin, has_output)

    def cloudkitty(self, action,
                   flags='', params='', fmt='-f json',
                   stdin=None, has_output=True):
        return self._run('cloudkitty', action, flags, params, fmt,
                         stdin, has_output)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.common import client
from cloudkittyclient.v1 import collector
from cloudkittyclient.v1 import info
from cloudkittyclient.v1 import rating
from cloudkittyclient.v1 import report
from cloudkittyclient.v1 import storage


class Client(client.BaseClient):

    def __init__(self,
                 session=None,
                 adapter_options={},
                 cacert=None,
                 insecure=False,
                 **kwargs):
        super(Client, self).__init__(
            session=session,
            adapter_options=adapter_options,
            cacert=cacert,
            insecure=insecure,
            **kwargs
        )

        self.info = info.InfoManager(self.api_client)
        self.collector = collector.CollectorManager(self.api_client)
        self.rating = rating.RatingManager(self.api_client)
        self.report = report.ReportManager(self.api_client)
        self.storage = storage.StorageManager(self.api_client)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import lister

from cloudkittyclient.common import base
from cloudkittyclient import exc
from cloudkittyclient import utils
from cloudkittyclient.v1.rating import hashmap
from cloudkittyclient.v1.rating import pyscripts


class RatingManager(base.BaseManager):
    """Class used to handle /v1/rating endpoint"""

    url = '/v1/rating/{endpoint}/{module_id}'

    def __init__(self, api_client):
        super(RatingManager, self).__init__(api_client)
        self.hashmap = hashmap.HashmapManager(api_client)
        self.pyscripts = pyscripts.PyscriptManager(api_client)

    def get_module(self, **kwargs):
        """Returns the given module.

        If module_id is not specified, returns the list of loaded modules.

        :param module_id: ID of the module on which you want information.
        :type module_id: str
        """
        authorized_args = ['module_id']
        url = self.get_url('modules', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def update_module(self, **kwargs):
        """Update the given module.

        :param module_id: Id of the module to update.
        :type module_id: str
        :param enabled: Set to True to enable the module, False to disable it.
        :type enabled: bool
        :param priority: New priority of the module.
        :type priority: int
        """
        if not kwargs.get('module_id', None):
            raise exc.ArgumentRequired("'module_id' argument is required.")
        url = self.get_url('modules', kwargs)
        module = self.get_module(**kwargs)
        for key in module.keys():
            value = kwargs.get(key, None)
            if value is not None and module[key] != value:
                module[key] = value
        self.api_client.put(url, json=module)
        return self.get_module(**kwargs)

    def reload_modules(self, **kwargs):
        """Triggers a reload of all rating modules."""
        url = self.get_url('reload_modules', kwargs)
        self.api_client.get(url)

    def get_quotation(self, **kwargs):
        """Returns a quote base on multiple resource descriptions.

        :param res_data: A list of resource descriptions.
        :type res_data: list
        """
        if not kwargs.get('res_data', None):
            raise exc.ArgumentRequired("'res_data' argument is required.")
        url = self.get_url('quote', {})

        body = {'resources': kwargs['res_data']}
        return self.api_client.post(url, json=body).json()


class CliModuleGet(lister.Lister):
    """Get a rating module or list loaded rating modules.

    If module_id is not specified, returns a list of all loaded
    rating modules.
    """
    columns = [
        ('module_id', 'Module'),
        ('enabled', 'Enabled'),
        ('priority', 'Priority'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.get_module(
            module_id=parsed_args.module_id,
        )
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliModuleGet, self).get_parser(prog_name)
        parser.add_argument('module_id', type=str, help='Module name')
        return parser


class CliModuleList(lister.Lister):
    """List loaded rating modules."""

    columns = [
        ('module_id', 'Module'),
        ('enabled', 'Enabled'),
        ('priority', 'Priority'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.get_module()
        values = utils.list_to_cols(resp['modules'], self.columns)
        return [col[1] for col in self.columns], values


class CliModuleSet(lister.Lister):
    columns = [
        ('module_id', 'Module'),
        ('enabled', 'Enabled'),
        ('priority', 'Priority'),
    ]

    def _take_action(self, **kwargs):
        resp = utils.get_client_from_osc(self).rating.update_module(**kwargs)
        values = [resp.get(col[0]) for col in self.columns]
        return [col[1] for col in self.columns], [values]

    def get_parser(self, prog_name):
        parser = super(CliModuleSet, self).get_parser(prog_name)
        parser.add_argument('module_id', type=str, help='Module name')
        return parser


class CliModuleEnable(CliModuleSet):
    """Enable a rating module."""

    def take_action(self, parsed_args):
        kwargs = vars(parsed_args)
        kwargs['enabled'] = True
        return self._take_action(**kwargs)


class CliModuleDisable(CliModuleEnable):
    """Disable a rating module."""

    def take_action(self, parsed_args):
        kwargs = vars(parsed_args)
        kwargs['enabled'] = False
        return self._take_action(**kwargs)


class CliModuleSetPriority(CliModuleSet):
    """Set the priority of a rating module."""

    def get_parser(self, prog_name):
        parser = super(CliModuleSetPriority, self).get_parser(prog_name)
        parser.add_argument('priority', type=int, help='Priority (int)')
        return parser

    def take_action(self, parsed_args):
        return self._take_action(**vars(parsed_args))



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from oslo_log import log

from cloudkittyclient.common import base
from cloudkittyclient import exc


LOG = log.getLogger(__name__)


class CollectorManager(base.BaseManager):
    """Class used to handle /v1/collector/mappings endpoint"""
    url = '/v1/collector/{endpoint}/{service_id}'

    def get_mapping(self, **kwargs):
        """Returns a service to collector mapping.

        If the service is not specified, returns a list of mappings for the
        given collector.

        :param service: Name of the service to filter on.
        :type service: str
        :param collector: Name of the collector to filter on.
        :type collector: str
        """
        LOG.warning('WARNING: Collector mappings are deprecated and will '
                    'be removed in a future release')
        kwargs['service_id'] = kwargs.get('service') or ''
        authorized_args = ['collector']
        url = self.get_url('mappings', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def create_mapping(self, **kwargs):
        """Creates a service to collector mapping.

        :param service: Name of the service to filter on.
        :type service: str
        :param collector: Name of the collector to filter on.
        :type collector: str
        """
        LOG.warning('WARNING: Collector mappings are deprecated and will '
                    'be removed in a future release')
        for arg in ('collector', 'service'):
            if not kwargs.get(arg):
                raise exc.ArgumentRequired(
                    "'{arg}' argument is required.".format(arg=arg))
        url = self.get_url('mappings', kwargs)
        body = dict(
            collector=kwargs['collector'],
            service=kwargs['service'])
        return self.api_client.post(url, json=body).json()

    def delete_mapping(self, **kwargs):
        """Deletes a service to collector mapping.

        :param service: Name of the service of which the mapping
                        should be deleted.
        :type service: str
        """
        LOG.warning('WARNING: Collector mappings are deprecated and will '
                    'be removed in a future release')
        if not kwargs.get('service'):
            raise exc.ArgumentRequired("'service' argument is required.")
        body = dict(service=kwargs['service'])
        url = self.get_url('mappings', kwargs)
        self.api_client.delete(url, json=body)

    def get_state(self, **kwargs):
        """Returns the state of a collector.

        :param name: Name of the collector.
        :type name: str
        """
        LOG.warning('WARNING: Collector mappings are deprecated and will '
                    'be removed in a future release')
        if not kwargs.get('name'):
            raise exc.ArgumentRequired("'name' argument is required.")
        authorized_args = ['name']
        url = self.get_url('states', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def set_state(self, **kwargs):
        """Sets the state of the collector.

        :param name: Name of the collector
        :type name: str
        :param enabled: State of the collector
        :type name: bool
        """
        LOG.warning('WARNING: Collector mappings are deprecated and will '
                    'be removed in a future release')
        if not kwargs.get('name'):
            raise exc.ArgumentRequired("'name' argument is required.")
        kwargs['enabled'] = kwargs.get('enabled') or False
        url = self.get_url('states', kwargs)
        body = dict(
            name=kwargs['name'],
            enabled=kwargs['enabled'],
        )
        self.api_client.put(url, json=body)
        return self.get_state(**kwargs)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.common import base


class StorageManager(base.BaseManager):
    """Class used to handle /v1/storage endpoint"""

    url = '/v1/storage/dataframes'

    def get_dataframes(self, **kwargs):
        """Returns a list of rated dataframes.

        :param begin: Begin timestamp
        :type begin: datetime
        :param end: End timestamp
        :type end: datetime
        :param tenant_id: ID of the tenant to filter on
        :type tenant_id: str
        :param resource_type: Resource type to filter on
        :type resource_type: str
        """
        authorized_args = ['begin', 'end', 'tenant_id', 'resource_type']
        url = self.get_url('', kwargs, authorized_args)
        return self.api_client.get(url).json()



# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from collections import abc
from unittest import mock

from cloudkittyclient.tests.unit.v1 import base
from cloudkittyclient.v1 import report_cli


class TestReportCli(base.BaseAPIEndpointTestCase):

    def test_report_tenant_list(self):

        class DummyAPIClient(object):
            def get_tenants(*args, **kwargs):
                return ['ee530dfc-319a-438f-9d43-346cfef501d6',
                        '91743a9a-688b-4526-b568-7b501531176c',
                        '4468704c-972e-4cfd-a342-9b71c493b79b']

        class ClientWrap(object):
            report = DummyAPIClient()

        class DummyParsedArgs(object):
            def __init__(self):
                self.begin = '2042-01-01T00:00:00'
                self.end = '2042-12-01T00:00:00'

        class DummyCliTenantList(report_cli.CliTenantList):
            def __init__(self):
                pass

        def __get_client_from_osc(*args):
            return ClientWrap()

        parsed_args = DummyParsedArgs()
        cli_class_instance = DummyCliTenantList()

        with mock.patch('cloudkittyclient.utils.get_client_from_osc',
                        new=__get_client_from_osc):
            # NOTE(peschk_l): self is only used used to get a client so just we
            # just override __init__ in order to skip class instanciation. In
            # python3 we could just have passed None
            result = report_cli.CliTenantList.take_action(
                cli_class_instance, parsed_args)

        assert len(result) == 2
        assert result[0] == ('Tenant ID', )
        assert isinstance(result[1], abc.Iterable)

        for res in result[1]:
            assert isinstance(res, abc.Iterable)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from keystoneauth1 import loading
from keystoneauth1 import plugin


class CloudKittyNoAuthPlugin(plugin.BaseAuthPlugin):
    """No authentication plugin for CloudKitty

    """
    def __init__(self, endpoint='http://localhost:8889', *args, **kwargs):
        super(CloudKittyNoAuthPlugin, self).__init__()
        self._endpoint = endpoint

    def get_auth_ref(self, session, **kwargs):
        return None

    def get_endpoint(self, session, **kwargs):
        return self._endpoint

    def get_headers(self, session, **kwargs):
        return {}


class CloudKittyNoAuthLoader(loading.BaseLoader):
    plugin_class = CloudKittyNoAuthPlugin

    def get_options(self):
        options = super(CloudKittyNoAuthLoader, self).get_options()
        options.extend([
            loading.Opt('endpoint', help='CloudKitty Endpoint',
                        required=True, default='http://localhost:8889'),
        ])
        return options



# Copyright 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from osc_lib import utils

DEFAULT_API_VERSION = '1'
API_VERSION_OPTION = 'os_rating_api_version'
API_NAME = "rating"
API_VERSIONS = {
    "1": "cloudkittyclient.v1.client.Client",
    "2": "cloudkittyclient.v2.client.Client",
}


def make_client(instance):
    """Returns a rating service client."""
    version = instance._api_version[API_NAME]
    ck_client = utils.get_client_class(
        API_NAME,
        version,
        API_VERSIONS)
    instance.setup_auth()
    adapter_options = dict(
        interface=instance.interface,
        region_name=instance.region_name,
    )
    return ck_client(session=instance.session,
                     adapter_options=adapter_options)


def build_option_parser(parser):
    """Hook to add global options."""
    parser.add_argument(
        '--rating-api-version', type=int, default=utils.env(
            'OS_RATING_API_VERSION',
            default=DEFAULT_API_VERSION)
    )
    return parser



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import json

from cloudkittyclient.common import base
from cloudkittyclient import exc


class DataframesManager(base.BaseManager):
    """Class used to handle /v2/dataframes endpoint"""

    url = '/v2/dataframes'

    def add_dataframes(self, **kwargs):
        """Add DataFrames to the storage backend. Returns nothing.

        :param dataframes: List of dataframes to add to the storage backend.
        :type dataframes: list of dataframes
        """

        dataframes = kwargs.get('dataframes')

        if not dataframes:
            raise exc.ArgumentRequired("'dataframes' argument is required")

        if not isinstance(dataframes, str):
            try:
                dataframes = json.dumps(dataframes)
            except TypeError:
                raise exc.InvalidArgumentError(
                    "'dataframes' must be either a string"
                    "or a JSON serializable object.")

        url = self.get_url(None, kwargs)
        return self.api_client.post(
            url,
            data=dataframes,
        )

    def get_dataframes(self, **kwargs):
        """Returns a paginated list of DataFrames.

        This support filters and datetime framing.

        :param offset: Index of the first dataframe that should be returned.
        :type offset: int
        :param limit: Maximal number of dataframes to return.
        :type limit: int
        :param filters: Optional dict of filters to select data on.
        :type filters: dict
        :param begin: Start of the period to gather data from
        :type begin: datetime.datetime
        :param end: End of the period to gather data from
        :type end: datetime.datetime
        """
        kwargs['filters'] = ','.join(
            '{}:{}'.format(k, v) for k, v in
            (kwargs.get('filters', None) or {}).items()
        )

        authorized_args = [
            'offset', 'limit', 'filters', 'begin', 'end']

        url = self.get_url(None, kwargs, authorized_args=authorized_args)
        return self.api_client.get(url).json()



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import os
from sys import argv

import cliff.app
from cliff.commandmanager import CommandManager
import os_client_config
from oslo_log import log

from cloudkittyclient import client
from cloudkittyclient.osc import DEFAULT_API_VERSION
from cloudkittyclient import utils


LOG = log.getLogger(__name__)


class CloudKittyShell(cliff.app.App):

    legacy_commands = [
        'module-list',
        'module-enable',
        'module-list',
        'module-enable',
        'module-disable',
        'module-set-priority',
        'info-config-get',
        'info-service-get',
        'total-get',
        'summary-get',
        'report-tenant-list',
        'collector-mapping-list',
        'collector-mapping-get',
        'collector-mapping-create',
        'collector-mapping-delete',
        'collector-state-get',
        'collector-state-enable',
        'collector-state-disable',
        'storage-dataframe-list',
        'hashmap-service-create',
        'hashmap-service-list',
        'hashmap-service-delete',
        'hashmap-field-create',
        'hashmap-field-list',
        'hashmap-field-delete',
        'hashmap-mapping-create',
        'hashmap-mapping-update',
        'hashmap-mapping-list',
        'hashmap-mapping-delete',
        'hashmap-group-create',
        'hashmap-group-list',
        'hashmap-group-delete',
        'hashmap-threshold-create'
        'hashmap-threshold-update'
        'hashmap-threshold-list',
        'hashmap-threshold-delete',
        'hashmap-threshold-get',
        'hashmap-threshold-group',
        'pyscripts-script-create',
        'pyscripts-script-list',
        'pyscripts-script-get',
        'pyscripts-script-get-data',
        'pyscripts-script-delete',
        'pyscripts-script-update',
    ]

    def _get_api_version(self, args):
        # FIXME(peschk_l): This is a hacky way to figure out the client version
        # to load. If anybody has a better idea, please fix this.
        self.deferred_help = True
        parser = self.build_option_parser('CloudKitty CLI client',
                                          utils.get_version())
        del self.deferred_help
        parsed_args = parser.parse_known_args(args)
        return str(parsed_args[0].os_rating_api_version or DEFAULT_API_VERSION)

    def __init__(self, args):
        self._args = args
        self.cloud_config = os_client_config.OpenStackConfig()
        super(CloudKittyShell, self).__init__(
            description='CloudKitty CLI client',
            version=utils.get_version(),
            command_manager=CommandManager('cloudkittyclient_v{}'.format(
                self._get_api_version(args[:]),
            )),
            deferred_help=True,
        )
        self._client = None

    # NOTE(peschk_l): Used to warn users about command syntax change in Rocky.
    # To be deleted in S.
    def run_subcommand(self, argv):
        try:
            self.command_manager.find_command(argv)
        except ValueError:
            if argv[0] in self.legacy_commands:
                LOG.warning('WARNING: This command is deprecated, please see'
                            ' the reference for the new commands\n')
                exit(1)
        return super(CloudKittyShell, self).run_subcommand(argv)

    def build_option_parser(self, description, version):
        parser = super(CloudKittyShell, self).build_option_parser(
            description,
            version,
            argparse_kwargs={'allow_abbrev': False})
        if 'OS_AUTH_TYPE' not in os.environ.keys() \
           and 'OS_PASSWORD' in os.environ.keys():
            os.environ['OS_AUTH_TYPE'] = 'password'
        self.cloud_config.register_argparse_arguments(
            parser, self._args, service_keys=['rating'])
        return parser

    @property
    def client(self):
        if self._client is None:
            self.cloud = self.cloud_config.get_one_cloud(
                argparse=self.options)
            session = self.cloud.get_session()
            adapter_options = dict(
                service_type=(self.options.os_rating_service_type or
                              self.options.os_service_type),
                service_name=(self.options.os_rating_service_name or
                              self.options.os_service_name),
                interface=(self.options.os_rating_interface or
                           self.options.os_interface),
                region_name=self.options.os_region_name,
                endpoint_override=(
                    self.options.os_rating_endpoint_override or
                    self.options.os_endpoint_override),
            )
            self._client = client.Client(
                str(self.options.os_rating_api_version or DEFAULT_API_VERSION),
                session=session,
                adapter_options=adapter_options)
        return self._client


def main(args=None):
    if args is None:
        args = argv[1:]
    client_app = CloudKittyShell(args)
    return client_app.run(args)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.unit.v1 import base


class TestInfo(base.BaseAPIEndpointTestCase):

    def test_get_metric(self):
        self.info.get_metric()
        self.api_client.get.assert_called_once_with('/v1/info/metrics/')

    def test_get_metric_with_arg(self):
        self.info.get_metric(metric_name='testmetric')
        self.api_client.get.assert_called_once_with(
            '/v1/info/metrics/testmetric')

    def test_get_config(self):
        self.info.get_config()
        self.api_client.get.assert_called_once_with('/v1/info/config/')



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.functional import base


class CkSummaryTest(base.BaseFunctionalTest):

    def __init__(self, *args, **kwargs):
        super(CkSummaryTest, self).__init__(*args, **kwargs)
        self.runner = self.cloudkitty

    def test_summary_get(self):
        return True
        # FIXME(peschk_l): Uncomment and update this once there is a way to set
        # the state of a summary through the client
        # resp = self.runner('summary get')


class OSCSummaryTest(CkSummaryTest):

    def __init__(self, *args, **kwargs):
        super(OSCSummaryTest, self).__init__(*args, **kwargs)
        self.runner = self.openstack



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.functional import base


class CkHashmapTest(base.BaseFunctionalTest):

    def __init__(self, *args, **kwargs):
        super(CkHashmapTest, self).__init__(*args, **kwargs)
        self.runner = self.cloudkitty

    def setUp(self):
        super(CkHashmapTest, self).setUp()
        self._fields = list()
        self._services = list()
        self._mappings = list()
        self._groups = list()
        self._thresholds = list()

    def tearDown(self):
        super(CkHashmapTest, self).tearDown()
        for field in self._fields:
            try:
                self.runner(
                    'hashmap field delete', params=field, has_output=False)
            except RuntimeError:
                pass
        for service in self._services:
            try:
                self.runner(
                    'hashmap service delete', params=service, has_output=False)
            except RuntimeError:
                pass
        for group in self._groups:
            try:
                self.runner(
                    'hashmap group delete', params=group, has_output=False)
            except RuntimeError:
                pass
        for mapping in self._mappings:
            try:
                self.runner(
                    'hashmap mapping delete', params=mapping, has_output=False)
            except RuntimeError:
                pass
        for threshold in self._thresholds:
            try:
                self.runner('hashmap threshold delete',
                            params=threshold, has_output=False)
            except RuntimeError:
                pass

    def test_list_mapping_types(self):
        resp = self.runner('hashmap mapping-types list')
        found_types = [elem['Mapping types'] for elem in resp]
        self.assertIn('flat', found_types)
        self.assertIn('rate', found_types)

    def test_create_get_delete_service(self):
        # Create service
        resp = self.runner('hashmap service create', params='testservice')[0]
        self.assertEqual(resp['Name'], 'testservice')
        service_id = resp['Service ID']
        self._services.append(service_id)

        # Check that resp is the same with service get and list
        resp_with_sid = self.runner(
            'hashmap service get', params=service_id)
        resp_without_sid = self.runner('hashmap service list')
        self.assertEqual(resp_with_sid, resp_without_sid)
        self.assertEqual(len(resp_with_sid), 1)

        # Check that deletion works
        self.runner('hashmap service delete',
                    params=resp['Service ID'],
                    has_output=False)
        resp = self.runner('hashmap service list')
        self.assertEqual(len(resp), 0)

    def test_group_get_create_delete(self):
        # Create group
        resp = self.runner('hashmap group create', params='testgroup')[0]
        self.assertEqual(resp['Name'], 'testgroup')
        group_id = resp['Group ID']
        self._groups.append(group_id)

        resp = self.runner('hashmap group list')
        self.assertEqual(len(resp), 1)

        # Check that deletion works
        self.runner('hashmap group delete',
                    params=group_id, has_output=False)
        resp = self.runner('hashmap group list')
        self.assertEqual(len(resp), 0)

    def test_create_get_delete_field(self):
        # Create service
        resp = self.runner('hashmap service create', params='testservice')[0]
        service_id = resp['Service ID']
        self._services.append(service_id)

        # Create field
        resp = self.runner('hashmap field create',
                           params='{} testfield'.format(service_id))[0]
        self.assertEqual(resp['Name'], 'testfield')
        self.assertEqual(resp['Service ID'], service_id)
        field_id = resp['Field ID']
        self._fields.append(field_id)

        # Check that resp is the same with field get and list
        resp_with_fid = self.runner('hashmap field get', params=field_id)
        resp_with_sid = self.runner('hashmap field list', params=service_id)
        self.assertEqual(resp_with_fid, resp_with_sid)
        self.assertEqual(len(resp_with_fid), 1)

        # Check that deletion works
        self.runner(
            'hashmap field delete', params=field_id, has_output=False)
        # resp = self.runner(
        #     'hashmap field list', params='-s {}'.format(service_id))
        resp = self.runner(
            'hashmap field list', params=service_id)
        self.assertEqual(len(resp), 0)

    def test_create_get_update_delete_mapping_service(self):
        resp = self.runner('hashmap service create', params='testservice')[0]
        service_id = resp['Service ID']
        self._services.append(service_id)

        # Create mapping
        resp = self.runner('hashmap mapping create',
                           params='-s {} 12'.format(service_id))[0]
        mapping_id = resp['Mapping ID']
        self._mappings.append(mapping_id)
        self.assertEqual(resp['Service ID'], service_id)
        self.assertEqual(float(resp['Cost']), float(12))

        # Get mapping
        resp_with_sid = self.runner(
            'hashmap mapping list', params='-s {}'.format(service_id))[0]
        resp_with_mid = self.runner(
            'hashmap mapping get', params=mapping_id)[0]
        self.assertEqual(resp_with_sid, resp_with_mid)
        self.assertEqual(resp_with_sid['Mapping ID'], mapping_id)
        self.assertEqual(resp_with_sid['Service ID'], service_id)
        self.assertEqual(float(resp_with_sid['Cost']), float(12))

        # Update mapping
        resp = self.runner('hashmap mapping update',
                           params='--cost 10 {}'.format(mapping_id))[0]
        self.assertEqual(float(resp['Cost']), float(10))

        # Check that deletion works
        self.runner(
            'hashmap mapping delete', params=mapping_id, has_output=False)
        resp = self.runner(
            'hashmap mapping list', params='-s {}'.format(service_id))
        self.assertEqual(len(resp), 0)
        self.runner(
            'hashmap service delete', params=service_id, has_output=False)

    def test_create_get_update_delete_mapping_field(self):
        resp = self.runner('hashmap service create', params='testservice')[0]
        service_id = resp['Service ID']
        self._services.append(service_id)

        resp = self.runner('hashmap field create',
                           params='{} testfield'.format(service_id))[0]
        field_id = resp['Field ID']
        self._fields.append(field_id)

        # Create mapping
        resp = self.runner(
            'hashmap mapping create',
            params='--field-id {} 12 --value testvalue'.format(field_id))[0]
        mapping_id = resp['Mapping ID']
        self._mappings.append(service_id)
        self.assertEqual(resp['Field ID'], field_id)
        self.assertEqual(float(resp['Cost']), float(12))
        self.assertEqual(resp['Value'], 'testvalue')

        # Get mapping
        resp = self.runner(
            'hashmap mapping get', params=mapping_id)[0]
        self.assertEqual(resp['Mapping ID'], mapping_id)
        self.assertEqual(float(resp['Cost']), float(12))

        # Update mapping
        resp = self.runner('hashmap mapping update',
                           params='--cost 10 {}'.format(mapping_id))[0]
        self.assertEqual(float(resp['Cost']), float(10))

    def test_group_mappings_get(self):
        # Service and group
        resp = self.runner('hashmap service create', params='testservice')[0]
        service_id = resp['Service ID']
        self._services.append(service_id)
        resp = self.runner('hashmap group create', params='testgroup')[0]
        group_id = resp['Group ID']
        self._groups.append(group_id)

        # Create service mapping bleonging to testgroup
        resp = self.runner(
            'hashmap mapping create',
            params='-s {} -g {} 12'.format(service_id, group_id))[0]
        mapping_id = resp['Mapping ID']
        self._mappings.append(mapping_id)

        resp = self.runner('hashmap group mappings get', params=group_id)[0]
        self.assertEqual(resp['Group ID'], group_id)
        self.assertEqual(float(resp['Cost']), float(12))

    def test_create_get_update_delete_threshold_service(self):
        resp = self.runner('hashmap service create', params='testservice')[0]
        service_id = resp['Service ID']
        self._services.append(service_id)

        # Create threshold
        resp = self.runner('hashmap threshold create',
                           params='-s {} 12 0.9'.format(service_id))[0]
        threshold_id = resp['Threshold ID']
        self._thresholds.append(threshold_id)
        self.assertEqual(resp['Service ID'], service_id)
        self.assertEqual(float(resp['Level']), float(12))
        self.assertEqual(float(resp['Cost']), float(0.9))

        # Get threshold
        resp_with_sid = self.runner(
            'hashmap threshold list', params='-s {}'.format(service_id))[0]
        resp_with_tid = self.runner(
            'hashmap threshold get', params=threshold_id)[0]
        self.assertEqual(resp_with_sid, resp_with_tid)
        self.assertEqual(resp_with_sid['Threshold ID'], threshold_id)
        self.assertEqual(resp_with_sid['Service ID'], service_id)
        self.assertEqual(float(resp_with_sid['Level']), float(12))
        self.assertEqual(float(resp_with_sid['Cost']), float(0.9))

        # Update threshold
        resp = self.runner('hashmap threshold update',
                           params='--cost 10 {}'.format(threshold_id))[0]
        self.assertEqual(float(resp['Cost']), float(10))

        # Check that deletion works
        self.runner(
            'hashmap threshold delete', params=threshold_id, has_output=False)
        resp = self.runner(
            'hashmap threshold list', params='-s {}'.format(service_id))
        self.assertEqual(len(resp), 0)

    def test_create_get_update_delete_threshold_field(self):
        resp = self.runner('hashmap service create', params='testservice')[0]
        service_id = resp['Service ID']
        self._services.append(service_id)

        resp = self.runner('hashmap field create',
                           params='{} testfield'.format(service_id))[0]
        field_id = resp['Field ID']
        self._fields.append(field_id)

        # Create threshold
        resp = self.runner(
            'hashmap threshold create',
            params='--field-id {} 12 0.9'.format(field_id))[0]
        threshold_id = resp['Threshold ID']
        self._thresholds.append(service_id)
        self.assertEqual(resp['Field ID'], field_id)
        self.assertEqual(float(resp['Level']), float(12))
        self.assertEqual(float(resp['Cost']), float(0.9))

        # Get threshold
        resp = self.runner('hashmap threshold get', params=threshold_id)[0]
        self.assertEqual(resp['Threshold ID'], threshold_id)
        self.assertEqual(float(resp['Level']), float(12))
        self.assertEqual(float(resp['Cost']), float(0.9))

        # Update threshold
        resp = self.runner('hashmap threshold update',
                           params='--cost 10 {}'.format(threshold_id))[0]
        self.assertEqual(float(resp['Cost']), float(10))

    def test_group_thresholds_get(self):
        # Service and group
        resp = self.runner('hashmap service create', params='testservice')[0]
        service_id = resp['Service ID']
        self._services.append(service_id)
        resp = self.runner('hashmap group create', params='testgroup')[0]
        group_id = resp['Group ID']
        self._groups.append(group_id)

        # Create service threshold bleonging to testgroup
        resp = self.runner(
            'hashmap threshold create',
            params='-s {} -g {} 12 0.9'.format(service_id, group_id))[0]
        threshold_id = resp['Threshold ID']
        self._thresholds.append(threshold_id)
        resp = self.runner('hashmap group thresholds get', params=group_id)[0]
        self.assertEqual(resp['Group ID'], group_id)
        self.assertEqual(float(resp['Level']), float(12))
        self.assertEqual(float(resp['Cost']), float(0.9))


class OSCHashmapTest(CkHashmapTest):

    def __init__(self, *args, **kwargs):
        super(OSCHashmapTest, self).__init__(*args, **kwargs)
        self.runner = self.openstack



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.common import base


class InfoManager(base.BaseManager):
    """Class used to handle /v1/info endpoint"""
    url = '/v1/info/{endpoint}/{metric_name}'

    def get_metric(self, **kwargs):
        """Returns info for the given service.

        If metric_name is not specified, returns info for all services.

        :param metric_name: Name of the service on which you want information
        :type metric_name: str
        """
        url = self.get_url('metrics', kwargs)
        return self.api_client.get(url).json()

    def get_config(self, **kwargs):
        """Returns the current configuration."""
        url = self.get_url('config', kwargs)
        return self.api_client.get(url).json()



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import command
from cliff import lister

from cloudkittyclient import utils


class BaseScriptCli(lister.Lister):

    columns = [
        ('name', 'Name'),
        ('script_id', 'Script ID'),
        ('checksum', 'Checksum'),
        ('data', 'Data'),
    ]


class CliGetScript(BaseScriptCli):
    """Get a PyScript."""

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.pyscripts.get_script(
            **vars(parsed_args))
        resp = [resp]
        values = utils.list_to_cols(resp, self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliGetScript, self).get_parser(prog_name)
        parser.add_argument('script_id', type=str, help='Script ID')
        return parser


class CliListScripts(BaseScriptCli):
    """List existing PyScripts."""

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.pyscripts.list_scripts(
            **vars(parsed_args))
        resp = resp.get('scripts') or []
        values = utils.list_to_cols(resp, self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliListScripts, self).get_parser(prog_name)
        parser.add_argument(
            '-n', '--no-data', action='store_true',
            help='Set to true to remove script data from output')
        return parser


class CliCreateScript(BaseScriptCli):
    """Create a PyScript."""

    def take_action(self, parsed_args):
        try:
            with open(parsed_args.data, 'r') as fd:
                parsed_args.data = fd.read()
        except IOError:
            pass
        resp = utils.get_client_from_osc(self).rating.pyscripts.create_script(
            **vars(parsed_args))
        resp = [resp]
        values = utils.list_to_cols(resp, self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCreateScript, self).get_parser(prog_name)
        parser.add_argument('name', type=str, help='Script Name')
        parser.add_argument('data', type=str, help='Script Data or data file')
        return parser


class CliUpdateScript(BaseScriptCli):
    """Update a PyScript."""

    def take_action(self, parsed_args):
        if parsed_args.data:
            try:
                with open(parsed_args.data, 'r') as fd:
                    parsed_args.data = fd.read()
            except IOError:
                pass
        resp = utils.get_client_from_osc(self).rating.pyscripts.update_script(
            **vars(parsed_args))
        resp = [resp]
        values = utils.list_to_cols(resp, self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliUpdateScript, self).get_parser(prog_name)
        parser.add_argument('script_id', type=str, help='Script ID')
        parser.add_argument('-n', '--name', type=str, help='Script Name')
        parser.add_argument('-d', '--data', type=str,
                            help='Script Data or data file')
        return parser


class CliDeleteScript(command.Command):
    """Delete a PyScript."""

    def take_action(self, parsed_args):
        utils.get_client_from_osc(self).rating.pyscripts.delete_script(
            **vars(parsed_args))

    def get_parser(self, prog_name):
        parser = super(CliDeleteScript, self).get_parser(prog_name)
        parser.add_argument('script_id', type=str, help='Script ID')
        return parser



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from collections import OrderedDict

from cloudkittyclient.tests.unit.v2 import base


class TestSummary(base.BaseAPIEndpointTestCase):

    def test_get_summary(self):
        self.summary.get_summary()
        self.api_client.get.assert_called_once_with('/v2/summary')

    def test_get_summary_with_pagination_args(self):
        self.summary.get_summary(offset=10, limit=10)
        try:
            self.api_client.get.assert_called_once_with(
                '/v2/summary?limit=10&offset=10')
        except AssertionError:
            self.api_client.get.assert_called_once_with(
                '/v2/summary?offset=10&limit=10')

    def test_get_summary_filters(self):
        self.summary.get_summary(
            filters=OrderedDict([('one', 'two'), ('three', 'four')]))
        self.api_client.get.assert_called_once_with(
            '/v2/summary?filters=one%3Atwo%2Cthree%3Afour')




# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient import exc
from cloudkittyclient.v1.client import rating


class RatingManager(rating.RatingManager):
    """Class used to handle /v2/rating/modules endpoint"""

    url = '/v2/rating/modules'

    def get_module(self, **kwargs):
        """Returns the given module.

        If module_id is not specified, returns the list of loaded modules.

        :param module_id: ID of the module on which you want information.
        :type module_id: str
        """
        module_id = kwargs.get('module_id', None)
        if module_id is not None:
            url = "{}/{}".format(self.url, module_id)
        else:
            url = self.url
        return self.api_client.get(url).json()

    def update_module(self, **kwargs):
        """Update the given module.

        :param module_id: Id of the module to update.
        :type module_id: str
        :param enabled: Set to True to enable the module, False to disable it.
        :type enabled: bool
        :param priority: New priority of the module.
        :type priority: int
        """
        if not kwargs.get('module_id', None):
            raise exc.ArgumentRequired("'module_id' argument is required.")
        mutable_fields = ['enabled', 'priority']
        changes = {}
        for key, value in kwargs.items():
            if value is not None and key in mutable_fields:
                changes[key] = value
        self.api_client.put("{}/{}".format(self.url, kwargs['module_id']),
                            json=changes)
        return self.get_module(**kwargs)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.functional import base


class CkPyscriptTest(base.BaseFunctionalTest):

    def __init__(self, *args, **kwargs):
        super(CkPyscriptTest, self).__init__(*args, **kwargs)
        self.runner = self.cloudkitty

    def test_create_get_update_list_delete(self):
        # Create
        resp = self.runner(
            'pyscript create', params="testscript 'return 0'")[0]
        script_id = resp['Script ID']
        self.assertEqual(resp['Name'], 'testscript')

        # Get
        resp = self.runner('pyscript get', params=script_id)[0]
        self.assertEqual(resp['Name'], 'testscript')
        self.assertEqual(resp['Script ID'], script_id)

        # Update
        resp = self.runner(
            'pyscript update',
            params="-n newname -d 'return 1' {}".format(script_id))[0]
        self.assertEqual(resp['Name'], 'newname')
        self.assertEqual(resp['Script ID'], script_id)
        self.assertEqual(resp['Data'], 'return 1')

        # List
        resp = self.runner('pyscript list')
        self.assertEqual(len(resp), 1)
        resp = resp[0]
        self.assertEqual(resp['Name'], 'newname')
        self.assertEqual(resp['Script ID'], script_id)
        self.assertEqual(resp['Data'], 'return 1')

        # Delete
        self.runner('pyscript delete', params=script_id, has_output=False)


class OSCPyscriptTest(CkPyscriptTest):

    def __init__(self, *args, **kwargs):
        super(CkPyscriptTest, self).__init__(*args, **kwargs)
        self.runner = self.openstack



# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import inspect
import sys

import pbr.version

from keystoneauth1.exceptions import http
from oslo_utils import timeutils


def get_version():
    """Returns cloudkittyclient's version."""
    return pbr.version.VersionInfo('python-cloudkittyclient').version_string()


def iso2dt(iso_date):
    """iso8601 format to datetime."""
    iso_dt = timeutils.parse_isotime(iso_date)
    trans_dt = timeutils.normalize_time(iso_dt)
    return trans_dt


def get_client_from_osc(obj):
    if hasattr(obj.app, 'client_manager'):
        return obj.app.client_manager.rating
    return obj.app.client


def dict_to_cols(dict_obj, cols):
    """Converts a dict to a cliff-compatible value list.

    For cliff lister.Lister objects, you should use list_to_cols() instead
    of this function.
    'cols' shouls be a list of (key, Name) tuples.
    """
    values = []
    for col in cols:
        values.append(dict_obj.get(col[0]))
    return values


def list_to_cols(list_obj, cols):
    if not isinstance(list_obj, list):
        list_obj = [list_obj]
    values = []
    for item in list_obj:
        values.append(dict_to_cols(item, cols))
    return values


def http_error_formatter(func):
    """This decorator catches Http Errors and re-formats them"""

    def wrap(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except http.HttpError as e:
            raise http.HttpError(message=e.response.text,
                                 http_status=e.http_status)

    return wrap


def format_http_errors(ignore):
    """Applies ``http_error_formatter`` to all methods of a class.

    :param ignore: List of function names to ignore
    :type ignore: iterable
    """

    def wrap(cls):
        # If you want pretty errors, use python3.
        # __qualname__ does not exist in python 2
        if sys.version_info.major < 3:
            return cls

        def predicate(item):
            # This avoids decorating functions of parent classes
            return (inspect.isfunction(item)
                    and item.__name__ not in ignore
                    and not item.__name__.startswith('_')
                    and cls.__name__ in item.__qualname__)

        for name, func in inspect.getmembers(cls, predicate):
            setattr(cls, name, http_error_formatter(func))

        return cls

    return wrap



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from cloudkittyclient.tests.unit.v2 import base


class TestRating(base.BaseAPIEndpointTestCase):

    def test_get_modules(self):
        self.rating.get_module()
        self.api_client.get.assert_called_once_with('/v2/rating/modules')

    def test_get_one_module(self):
        self.rating.get_module(module_id="moduleidtest")
        self.api_client.get.assert_called_once_with(
            '/v2/rating/modules/moduleidtest')

    def test_update_one_module(self):
        self.rating.update_module(module_id="moduleidtest",
                                  enabled=False, priority=42)
        self.api_client.put.assert_called_once_with(
            '/v2/rating/modules/moduleidtest',
            json={
                'enabled': False,
                'priority': 42,
            })



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import csv

from cliff.formatters import base
import jsonpath_rw_ext as jp
from oslo_log import log
import yaml


LOG = log.getLogger(__name__)


class DataframeToCsvFormatter(base.ListFormatter):
    """Cliff formatter allowing to customize CSV report content."""

    default_config = [
        ('Begin', '$.begin'),
        ('End', '$.end'),
        ('Metric Type', '$.service'),
        ('Qty', '$.volume'),
        ('Cost', '$.rating'),
        ('Project ID', '$.desc.project_id'),
        ('Resource ID', '$.desc.resource_id'),
        ('User ID', '$.desc.user_id'),
    ]

    def _load_config(self, filename):
        config = self.default_config
        if filename:
            try:
                with open(filename, 'r') as fd:
                    yml_config = yaml.safe_load(fd.read())
                if len(yml_config):
                    config = [(list(item.keys())[0], list(item.values())[0])
                              for item in yml_config]
                else:
                    LOG.warning('Invalid config file {file}. Using default '
                                'configuration'.format(file=filename))
            except (IOError, yaml.scanner.ScannerError) as err:
                LOG.warning('Error: {err}. Using default '
                            'configuration'.format(err=err))
        self.parsers = {}
        for col, path in config:
            self.parsers[col] = jp.parse(path)
        return config

    def add_argument_group(self, parser):
        group = parser.add_argument_group('dataframe-to-csv formatter')
        group.add_argument('--format-config-file',
                           type=str, dest='format_config',
                           help='Config file for the dict-to-csv formatter')

    def _get_csv_row(self, config, json_item):
        row = {}
        for col, parser in self.parsers.items():
            items = parser.find(json_item)
            row[col] = items[0].value if items else ''
        return row

    def emit_list(self, column_names, data, stdout, parsed_args):
        config = self._load_config(vars(parsed_args).get('format_config'))
        self.writer = csv.DictWriter(stdout,
                                     fieldnames=[elem[0] for elem in config])
        self.writer.writeheader()
        for dataframe in data:
            rating_data = dataframe[3]
            for item in rating_data:
                item['begin'] = dataframe[0]
                item['end'] = dataframe[1]
                row = self._get_csv_row(config, item)
                self.writer.writerow(row)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import lister
from cliff import show

from cloudkittyclient import exc
from cloudkittyclient import utils


class CliSummaryGet(lister.Lister):
    """Get a summary report."""
    summary_columns = [
        ('tenant_id', 'Tenant ID'),
        ('res_type', 'Resource Type'),
        ('rate', 'Rate'),
        ('begin', 'Begin Time'),
        ('end', 'End Time'),
    ]

    def take_action(self, parsed_args):
        for arg in ['begin', 'end']:
            value = getattr(parsed_args, arg)
            if value is not None:
                try:
                    setattr(parsed_args, arg, utils.iso2dt(value))
                except ValueError:
                    raise exc.InvalidArgumentError(
                        'Invalid timestamp "{}"'.format(value))
        resp = utils.get_client_from_osc(self).report.get_summary(
            **vars(parsed_args))
        values = utils.list_to_cols(
            resp.get('summary', []), self.summary_columns)
        return [col[1] for col in self.summary_columns], values

    def get_parser(self, prog_name):
        parser = super(CliSummaryGet, self).get_parser(prog_name)
        parser.add_argument('-t', '--tenant-id', type=str,
                            help='Tenant id.')
        parser.add_argument('-b', '--begin', type=str,
                            help='Begin timestamp.')
        parser.add_argument('-e', '--end', type=str,
                            help='End timestamp.')
        parser.add_argument('-s', '--service', type=str,
                            help='Service Type.')
        parser.add_argument('-g', '--groupby', nargs='+',
                            help='Fields to group by, space-separated. '
                            '(res_type and tenant_id are supported for now)')
        parser.add_argument('-a', '--all-tenants', action='store_true',
                            help='Allows to get summary from all tenants '
                            '(admin only). Defaults to False.')
        return parser


class CliTotalGet(show.ShowOne):
    """(DEPRECATED) Get total reports."""
    def take_action(self, parsed_args):
        for arg in ['begin', 'end']:
            value = getattr(parsed_args, arg)
            if value is not None:
                try:
                    setattr(parsed_args, arg, utils.iso2dt(value))
                except ValueError:
                    raise exc.InvalidArgumentError(
                        'Invalid timestamp "{}"'.format(value))
        resp = utils.get_client_from_osc(self).report.get_total(
            **vars(parsed_args))
        return ('Total', ), (float(resp), )

    def get_parser(self, prog_name):
        parser = super(CliTotalGet, self).get_parser(prog_name)
        parser.add_argument('-t', '--tenant-id',
                            help='Tenant id.')
        parser.add_argument('-b', '--begin', type=str,
                            help='Begin timestamp.')
        parser.add_argument('-e', '--end', type=str,
                            help='End timestamp.')
        parser.add_argument('-s', '--service',
                            help='Service Type.')
        parser.add_argument('-g', '--groupby', nargs='+',
                            help='Fields to group by, space-separated. '
                            '(res_type and tenant_id are supported for now)')
        parser.add_argument('-a', '--all-tenants', action='store_true',
                            help='Allows to get summary from all tenants '
                            '(admin only). Defaults to False.')
        return parser


class CliTenantList(lister.Lister):
    """Get rated tenants for the given period.

    Begin defaults to the beginning of the current month and end defaults to
    the beginning of the next month.
    """
    def take_action(self, parsed_args):
        for arg in ['begin', 'end']:
            value = getattr(parsed_args, arg)
            if value is not None:
                try:
                    setattr(parsed_args, arg, utils.iso2dt(value))
                except ValueError:
                    raise exc.InvalidArgumentError(
                        'Invalid timestamp "{}"'.format(value))
        client = utils.get_client_from_osc(self)
        tenants = client.report.get_tenants(**vars(parsed_args))
        output = []
        for tenant in tenants:
            output.append((tenant, ))
        return (('Tenant ID', ), output)

    def get_parser(self, prog_name):
        parser = super(CliTenantList, self).get_parser(prog_name)
        parser.add_argument('-b', '--begin', type=str,
                            help='Begin timestamp.')
        parser.add_argument('-e', '--end', type=str,
                            help='End timestamp.')
        return parser



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import os
import uuid

from cloudkittyclient.tests.functional import base


class CkDataframesTest(base.BaseFunctionalTest):
    dataframes_data = """
    {
        "dataframes": [
            {
                "period": {
                    "begin": "20190723T122810Z",
                    "end": "20190723T132810Z"
                },
                "usage": {
                    "metric_one": [
                        {
                            "vol": {
                                "unit": "GiB",
                                "qty": 1.2
                            },
                            "rating": {
                                "price": 0.04
                            },
                            "groupby": {
                                "group_one": "one",
                                "group_two": "two"
                            },
                            "metadata": {
                                "attr_one": "one",
                                "attr_two": "two"
                            }
                        }
                    ],
                    "metric_two": [
                        {
                            "vol": {
                                "unit": "MB",
                                "qty": 200.4
                            },
                            "rating": {
                                "price": 0.06
                            },
                            "groupby": {
                                "group_one": "one",
                                "group_two": "two"
                            },
                            "metadata": {
                                "attr_one": "one",
                                "attr_two": "two"
                            }
                        }
                    ]
                }
            },
            {
                "period": {
                    "begin": "20190823T122810Z",
                    "end": "20190823T132810Z"
                },
                "usage": {
                    "metric_one": [
                        {
                            "vol": {
                                "unit": "GiB",
                                "qty": 2.4
                            },
                            "rating": {
                                "price": 0.08
                            },
                            "groupby": {
                                "group_one": "one",
                                "group_two": "two"
                            },
                            "metadata": {
                                "attr_one": "one",
                                "attr_two": "two"
                            }
                        }
                    ],
                    "metric_two": [
                        {
                            "vol": {
                                "unit": "MB",
                                "qty": 400.8
                            },
                            "rating": {
                                "price": 0.12
                            },
                            "groupby": {
                                "group_one": "one",
                                "group_two": "two"
                            },
                            "metadata": {
                                "attr_one": "one",
                                "attr_two": "two"
                            }
                        }
                    ]
                }
            }
        ]
    }
    """

    def __init__(self, *args, **kwargs):
        super(CkDataframesTest, self).__init__(*args, **kwargs)
        self.runner = self.cloudkitty

    def setUp(self):
        super(CkDataframesTest, self).setUp()

        self.fixture_file_name = '{}.json'.format(uuid.uuid4())
        with open(self.fixture_file_name, 'w') as f:
            f.write(self.dataframes_data)

    def tearDown(self):
        files = os.listdir('.')
        if self.fixture_file_name in files:
            os.remove(self.fixture_file_name)

        super(CkDataframesTest, self).tearDown()

    def test_dataframes_add_with_no_args(self):
        self.assertRaisesRegexp(
            RuntimeError,
            'error: the following arguments are required: datafile',
            self.runner,
            'dataframes add',
            fmt='',
            has_output=False,
        )

    def test_dataframes_add(self):
        self.runner(
            'dataframes add {}'.format(self.fixture_file_name),
            fmt='',
            has_output=False,
        )

    def test_dataframes_add_with_hyphen_stdin(self):
        with open(self.fixture_file_name, 'r') as f:
            self.runner(
                'dataframes add -',
                fmt='',
                stdin=f.read().encode(),
                has_output=False,
            )

    def test_dataframes_get(self):
        # TODO(jferrieu): functional tests will be added in another
        # patch for `dataframes get`
        pass


class OSCDataframesTest(CkDataframesTest):
    def __init__(self, *args, **kwargs):
        super(OSCDataframesTest, self).__init__(*args, **kwargs)
        self.runner = self.openstack



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient import exc
from cloudkittyclient.tests.unit.v1 import base


class TestCollector(base.BaseAPIEndpointTestCase):

    def test_get_mapping_no_args(self):
        self.collector.get_mapping()
        self.api_client.get.assert_called_once_with('/v1/collector/mappings/')

    def test_get_mapping_service_id(self):
        self.collector.get_mapping(service='testservice')
        self.api_client.get.assert_called_once_with(
            '/v1/collector/mappings/testservice')

    def test_get_mapping_collector(self):
        self.collector.get_mapping(collector='testcollector')
        self.api_client.get.assert_called_once_with(
            '/v1/collector/mappings/?collector=testcollector')

    def test_get_mapping_collector_service_id(self):
        self.collector.get_mapping(
            service='testservice', collector='testcollector')
        self.api_client.get.assert_called_once_with(
            '/v1/collector/mappings/testservice?collector=testcollector')

    def test_create_mapping(self):
        kwargs = dict(service='testservice', collector='testcollector')
        self.collector.create_mapping(**kwargs)
        self.api_client.post.assert_called_once_with(
            '/v1/collector/mappings/', json=kwargs)

    def test_create_mapping_no_name(self):
        self.assertRaises(exc.ArgumentRequired,
                          self.collector.create_mapping,
                          collector='testcollector')

    def test_delete_mapping(self):
        kwargs = dict(service='testservice')
        self.collector.delete_mapping(**kwargs)
        self.api_client.delete.assert_called_once_with(
            '/v1/collector/mappings/', json=kwargs)

    def test_delete_mapping_no_service(self):
        self.assertRaises(exc.ArgumentRequired,
                          self.collector.create_mapping)

    def test_get_state(self):
        self.collector.get_state(name='testcollector')
        self.api_client.get.assert_called_once_with(
            '/v1/collector/states/?name=testcollector')

    def test_set_state(self):
        kwargs = dict(name='testcollector', enabled=True)
        self.collector.set_state(**kwargs)
        self.api_client.put.assert_called_once_with(
            '/v1/collector/states/', json=kwargs)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from unittest import mock

from cloudkittyclient import exc
from cloudkittyclient.tests.unit.v1 import base
from cloudkittyclient.tests import utils


class TestHashmap(base.BaseAPIEndpointTestCase):

    def test_get_mapping_types(self):
        self.hashmap.get_mapping_types()
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/types/')

    def test_get_service(self):
        self.hashmap.get_service()
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/services/')

    def test_get_service_service_id(self):
        self.hashmap.get_service(service_id='service_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/services/service_id')

    def test_create_service(self):
        kwargs = dict(name='service')
        self.hashmap.create_service(**kwargs)
        self.api_client.post.assert_called_once_with(
            '/v1/rating/module_config/hashmap/services/', json=kwargs)

    def test_create_service_no_name(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.create_service)

    def test_delete_service(self):
        self.hashmap.delete_service(service_id='service_id')
        self.api_client.delete.assert_called_once_with(
            '/v1/rating/module_config/hashmap/services/',
            json={'service_id': 'service_id'})

    def test_delete_service_no_id(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.delete_service)

    def test_get_fields_of_service(self):
        self.hashmap.get_field(service_id='service_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/fields/?service_id=service_id')

    def test_get_field(self):
        self.hashmap.get_field(field_id='field_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/fields/field_id')

    def test_get_field_no_args(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.get_field)

    def test_get_field_with_service_id_and_field_id(self):
        self.assertRaises(exc.InvalidArgumentError, self.hashmap.get_field,
                          service_id='service_id', field_id='field_id')

    def test_create_field(self):
        kwargs = dict(name='name', service_id='service_id')
        self.hashmap.create_field(**kwargs)
        self.api_client.post.assert_called_once_with(
            '/v1/rating/module_config/hashmap/fields/', json=kwargs)

    def test_create_field_no_name(self):
        self.assertRaises(exc.ArgumentRequired,
                          self.hashmap.create_field,
                          service_id='service_id')

    def test_create_field_no_service_id(self):
        self.assertRaises(
            exc.ArgumentRequired, self.hashmap.create_field, name='name')

    def test_delete_field(self):
        kwargs = dict(field_id='field_id')
        self.hashmap.delete_field(**kwargs)
        self.api_client.delete.assert_called_once_with(
            '/v1/rating/module_config/hashmap/fields/', json=kwargs)

    def test_delete_field_no_arg(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.delete_field)

    def test_get_mapping_with_id(self):
        self.hashmap.get_mapping(mapping_id='mapping_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/mappings/mapping_id')

    def test_get_mapping_service_id(self):
        self.hashmap.get_mapping(service_id='service_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/mappings/?service_id=service_id')

    def test_get_mapping_no_args(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.get_mapping)

    def test_create_mapping(self):
        kwargs = dict(cost=2, value='value', field_id='field_id')
        body = dict(
            cost=kwargs.get('cost'),
            value=kwargs.get('value'),
            service_id=kwargs.get('service_id'),
            field_id=kwargs.get('field_id'),
            group_id=kwargs.get('group_id'),
            tenant_id=kwargs.get('tenant_id'),
            type=kwargs.get('type') or 'flat',
        )
        self.hashmap.create_mapping(**kwargs)
        self.api_client.post.assert_called_once_with(
            '/v1/rating/module_config/hashmap/mappings/', json=body)

    def test_create_mapping_no_cost(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.create_mapping,
                          value='value', field_id='field_id')

    def test_create_mapping_no_id(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.create_mapping,
                          value='value', cost=12)

    def test_create_mapping_field_and_service_id(self):
        self.assertRaises(
            exc.InvalidArgumentError, self.hashmap.create_mapping, cost=12,
            field_id='field_id', service_id='service_id')

    def test_create_mapping_value_and_service_id(self):
        self.assertRaises(
            exc.InvalidArgumentError, self.hashmap.create_mapping,
            value='value', service_id='service_id', cost=0.8)

    def test_update_mapping(self):
        kwargs = dict(
            cost=12,
            value='value',
            service_id='service_id',
            field_id='field_id',
            tenant_id='tenant_id',
            type='type',
            mapping_id='mapping_id',
        )
        fake_get = mock.Mock(return_value=utils.FakeRequest(
            cost='Bad value',
            value='Bad value',
            service_id='Bad value',
            field_id='Bad value',
            tenant_id='Bad value',
            type='Bad value',
            mapping_id='mapping_id',
        ))
        self.api_client.get = fake_get
        self.hashmap.update_mapping(**kwargs)
        self.api_client.get.assert_called_with(
            '/v1/rating/module_config/hashmap/mappings/mapping_id')
        self.api_client.put.assert_called_once_with(
            '/v1/rating/module_config/hashmap/mappings/', json=kwargs)

    def test_update_mapping_no_arg(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.update_mapping)

    def test_get_mapping_group(self):
        self.hashmap.get_mapping_group(mapping_id='mapping_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/'
            'hashmap/mappings/group?mapping_id=mapping_id')

    def test_delete_mapping(self):
        kwargs = dict(mapping_id='mapping_id')
        self.hashmap.delete_mapping(**kwargs)
        self.api_client.delete.assert_called_once_with(
            '/v1/rating/module_config/hashmap/mappings/', json=kwargs)

    def test_delete_mapping_no_arg(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.delete_mapping)

    def test_get_mapping_group_no_arg(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.get_mapping_group)

    def test_get_group_no_arg(self):
        self.hashmap.get_group()
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/groups/')

    def test_get_group(self):
        self.hashmap.get_group(group_id='group_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/groups/group_id')

    def test_create_group(self):
        kwargs = dict(name='group')
        self.hashmap.create_group(**kwargs)
        self.api_client.post.assert_called_once_with(
            '/v1/rating/module_config/hashmap/groups/',
            json=kwargs)

    def test_create_group_no_name(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.create_group)

    def test_delete_group(self):
        kwargs = dict(group_id='group_id')
        self.hashmap.delete_group(**kwargs)
        kwargs['recursive'] = False
        self.api_client.delete.assert_called_once_with(
            '/v1/rating/module_config/hashmap/groups/',
            json=kwargs)

    def test_delete_group_recursive(self):
        kwargs = dict(group_id='group_id', recursive=True)
        self.hashmap.delete_group(**kwargs)
        self.api_client.delete.assert_called_once_with(
            '/v1/rating/module_config/hashmap/groups/',
            json=kwargs)

    def test_delete_group_no_id(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.create_group)

    def test_get_group_mappings(self):
        self.hashmap.get_group_mappings(group_id='group_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/groups/mappings'
            '?group_id=group_id')

    def test_get_group_mappings_no_args(self):
        self.assertRaises(
            exc.ArgumentRequired, self.hashmap.get_group_mappings)

    def test_get_group_thresholds(self):
        self.hashmap.get_group_thresholds(group_id='group_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/groups/thresholds'
            '?group_id=group_id')

    def test_get_group_thresholds_no_args(self):
        self.assertRaises(
            exc.ArgumentRequired, self.hashmap.get_group_thresholds)

    def test_get_threshold_with_id(self):
        self.hashmap.get_threshold(threshold_id='threshold_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/thresholds/threshold_id')

    def test_get_threshold_service_id(self):
        self.hashmap.get_threshold(service_id='service_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/hashmap/thresholds/'
            '?service_id=service_id')

    def test_get_threshold_no_args(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.get_threshold)

    def test_create_threshold(self):
        kwargs = dict(cost=2, level=123, field_id='field_id')
        body = dict(
            cost=kwargs.get('cost'),
            level=kwargs.get('level'),
            service_id=kwargs.get('service_id'),
            field_id=kwargs.get('field_id'),
            group_id=kwargs.get('group_id'),
            tenant_id=kwargs.get('tenant_id'),
            type=kwargs.get('type') or 'flat',
        )
        self.hashmap.create_threshold(**kwargs)
        self.api_client.post.assert_called_once_with(
            '/v1/rating/module_config/hashmap/thresholds/', json=body)

    def test_create_threshold_no_cost(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.create_threshold,
                          level=123, field_id='field_id')

    def test_create_threshold_no_id(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.create_threshold,
                          level=123, cost=12)

    def test_create_threshold_field_and_service_id(self):
        self.assertRaises(
            exc.ArgumentRequired, self.hashmap.create_threshold, cost=12,
            field_id='field_id', service_id='service_id')

    def test_delete_threshold(self):
        kwargs = dict(threshold_id='threshold_id')
        self.hashmap.delete_threshold(**kwargs)
        self.api_client.delete.assert_called_once_with(
            '/v1/rating/module_config/hashmap/thresholds/', json=kwargs)

    def test_delete_threshold_no_arg(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.delete_threshold)

    def test_update_threshold(self):
        kwargs = dict(
            cost=12,
            level=123,
            service_id='service_id',
            field_id='field_id',
            tenant_id='tenant_id',
            type='type',
            threshold_id='threshold_id'
        )
        fake_get = mock.Mock(return_value=utils.FakeRequest(
            cost='Bad value',
            level='Bad value',
            service_id='Bad value',
            field_id='Bad value',
            tenant_id='Bad value',
            type='Bad value',
            threshold_id='threshold_id'
        ))
        self.api_client.get = fake_get
        self.hashmap.update_threshold(**kwargs)
        self.api_client.get.assert_called_with(
            '/v1/rating/module_config/hashmap/thresholds/threshold_id')
        self.api_client.put.assert_called_once_with(
            '/v1/rating/module_config/hashmap/thresholds/', json=kwargs)

    def test_update_threshold_no_arg(self):
        self.assertRaises(exc.ArgumentRequired, self.hashmap.update_threshold)

    def test_get_threshold_group(self):
        self.hashmap.get_threshold_group(threshold_id='threshold_id')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/'
            'hashmap/thresholds/group?threshold_id=threshold_id')

    def test_get_threshold_group_no_arg(self):
        self.assertRaises(
            exc.ArgumentRequired, self.hashmap.get_threshold_group)



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import json

from collections import OrderedDict

from cloudkittyclient import exc
from cloudkittyclient.tests.unit.v2 import base


class TestDataframes(base.BaseAPIEndpointTestCase):
    dataframes_data = """
    {
        "dataframes": [
            {
                "period": {
                    "begin": "20190723T122810Z",
                    "end": "20190723T132810Z"
                },
                "usage": {
                    "metric_one": [
                        {
                            "vol": {
                                "unit": "GiB",
                                "qty": 1.2
                            },
                            "rating": {
                                "price": 0.04
                            },
                            "groupby": {
                                "group_one": "one",
                                "group_two": "two"
                            },
                            "metadata": {
                                "attr_one": "one",
                                "attr_two": "two"
                            }
                        }
                    ],
                    "metric_two": [
                        {
                            "vol": {
                                "unit": "MB",
                                "qty": 200.4
                            },
                            "rating": {
                                "price": 0.06
                            },
                            "groupby": {
                                "group_one": "one",
                                "group_two": "two"
                            },
                            "metadata": {
                                "attr_one": "one",
                                "attr_two": "two"
                            }
                        }
                    ]
                }
            },
            {
                "period": {
                    "begin": "20190823T122810Z",
                    "end": "20190823T132810Z"
                },
                "usage": {
                    "metric_one": [
                        {
                            "vol": {
                                "unit": "GiB",
                                "qty": 2.4
                            },
                            "rating": {
                                "price": 0.08
                            },
                            "groupby": {
                                "group_one": "one",
                                "group_two": "two"
                            },
                            "metadata": {
                                "attr_one": "one",
                                "attr_two": "two"
                            }
                        }
                    ],
                    "metric_two": [
                        {
                            "vol": {
                                "unit": "MB",
                                "qty": 400.8
                            },
                            "rating": {
                                "price": 0.12
                            },
                            "groupby": {
                                "group_one": "one",
                                "group_two": "two"
                            },
                            "metadata": {
                                "attr_one": "one",
                                "attr_two": "two"
                            }
                        }
                    ]
                }
            }
        ]
    }
    """

    def test_add_dataframes_with_string(self):
        self.dataframes.add_dataframes(
            dataframes=self.dataframes_data,
        )
        self.api_client.post.assert_called_once_with(
            '/v2/dataframes',
            data=self.dataframes_data,
        )

    def test_add_dataframes_with_json_object(self):
        json_data = json.loads(self.dataframes_data)

        self.dataframes.add_dataframes(
            dataframes=json_data,
        )
        self.api_client.post.assert_called_once_with(
            '/v2/dataframes',
            data=json.dumps(json_data),
        )

    def test_add_dataframes_with_neither_string_nor_object_raises_exc(self):
        self.assertRaises(
            exc.InvalidArgumentError,
            self.dataframes.add_dataframes,
            dataframes=[open],
        )

    def test_add_dataframes_with_no_args_raises_exc(self):
        self.assertRaises(
            exc.ArgumentRequired,
            self.dataframes.add_dataframes)

    def test_get_dataframes(self):
        self.dataframes.get_dataframes()
        self.api_client.get.assert_called_once_with('/v2/dataframes')

    def test_get_dataframes_with_pagination_args(self):
        self.dataframes.get_dataframes(offset=10, limit=10)
        try:
            self.api_client.get.assert_called_once_with(
                '/v2/dataframes?limit=10&offset=10')
        except AssertionError:
            self.api_client.get.assert_called_once_with(
                '/v2/dataframes?offset=10&limit=10')

    def test_get_dataframes_filters(self):
        self.dataframes.get_dataframes(
            filters=OrderedDict([('one', 'two'), ('three', 'four')]))
        self.api_client.get.assert_called_once_with(
            '/v2/dataframes?filters=one%3Atwo%2Cthree%3Afour')



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cloudkittyclient.tests import utils
from cloudkittyclient.v1 import collector
from cloudkittyclient.v1 import info
from cloudkittyclient.v1 import rating
from cloudkittyclient.v1.rating import hashmap
from cloudkittyclient.v1.rating import pyscripts
from cloudkittyclient.v1 import report
from cloudkittyclient.v1 import storage


class BaseAPIEndpointTestCase(utils.BaseTestCase):

    def setUp(self):
        super(BaseAPIEndpointTestCase, self).setUp()
        self.api_client = utils.FakeHTTPClient()
        self.storage = storage.StorageManager(self.api_client)
        self.rating = rating.RatingManager(self.api_client)
        self.collector = collector.CollectorManager(self.api_client)
        self.info = info.InfoManager(self.api_client)
        self.report = report.ReportManager(self.api_client)
        self.pyscripts = pyscripts.PyscriptManager(self.api_client)
        self.hashmap = hashmap.HashmapManager(self.api_client)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import command
from cliff import lister

from cloudkittyclient import utils


class CliGetMappingTypes(lister.Lister):
    """Get hashmap mapping types/"""
    def take_action(self, parsed_args):
        client = utils.get_client_from_osc(self)
        resp = client.rating.hashmap.get_mapping_types()
        return ['Mapping types'], [[item] for item in resp]


class CliGetService(lister.Lister):
    """Get a hashmap service"""

    columns = [
        ('name', 'Name'),
        ('service_id', 'Service ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_service(
            service_id=parsed_args.service_id,
        )
        # NOTE(lukapeschke): This can't be done with 'or', because it would
        # lead to resp being [[]] if resp['services'] is an empty list. Having
        # a list in a list causes cliff to display a row of 'None' instead of
        # nothing
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliGetService, self).get_parser(prog_name)
        parser.add_argument('service_id', type=str, help='Service ID')
        return parser


class CliListService(lister.Lister):
    """List hashmap services."""

    columns = [
        ('name', 'Name'),
        ('service_id', 'Service ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_service()
        # NOTE(lukapeschke): This can't be done with 'or', because it would
        # lead to resp being [[]] if resp['services'] is an empty list. Having
        # a list in a list causes cliff to display a row of 'None' instead of
        # nothing
        values = utils.list_to_cols(resp['services'], self.columns)
        return [col[1] for col in self.columns], values


class CliCreateService(lister.Lister):
    """Create a hashmap service."""

    columns = [
        ('name', 'Name'),
        ('service_id', 'Service ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.create_service(
            **vars(parsed_args))
        values = utils.list_to_cols(resp, self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCreateService, self).get_parser(prog_name)
        parser.add_argument('name', type=str, help='Service Name')
        return parser


class CliDeleteService(command.Command):
    """Delete a hashmap service"""

    def take_action(self, parsed_args):
        utils.get_client_from_osc(self).rating.hashmap.delete_service(
            **vars(parsed_args))

    def get_parser(self, prog_name):
        parser = super(CliDeleteService, self).get_parser(prog_name)
        parser.add_argument('service_id', type=str, help='Service ID')
        return parser


class CliGetField(lister.Lister):
    """Get a Hashmap field."""
    columns = [
        ('name', 'Name'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_field(
            field_id=parsed_args.field_id,
        )
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliGetField, self).get_parser(prog_name)
        parser.add_argument('field_id', type=str, help='Field ID')
        return parser


class CliListField(lister.Lister):
    """List hashmap fields for the given service."""

    columns = [
        ('name', 'Name'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_field(
            service_id=parsed_args.service_id,
        )
        values = utils.list_to_cols(resp['fields'], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliListField, self).get_parser(prog_name)
        parser.add_argument('service_id', type=str, help='Service ID')
        return parser


class CliCreateField(lister.Lister):
    """Create a hashmap field."""
    columns = [
        ('name', 'Name'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.create_field(
            **vars(parsed_args))
        resp = [resp] if resp.get('fields') is None else resp['fields']
        values = utils.list_to_cols(resp, self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCreateField, self).get_parser(prog_name)
        parser.add_argument('service_id', type=str, help='Service ID')
        parser.add_argument('name', type=str, help='Field name')
        return parser


class CliDeleteField(command.Command):
    """Delete a hashmap field."""

    def take_action(self, parsed_args):
        utils.get_client_from_osc(self).rating.hashmap.delete_field(
            **vars(parsed_args))

    def get_parser(self, prog_name):
        parser = super(CliDeleteField, self).get_parser(prog_name)
        parser.add_argument('field_id', type=str, help='Field ID')
        return parser


class CliGetMapping(lister.Lister):
    """Get a hashmap mapping."""

    columns = [
        ('mapping_id', 'Mapping ID'),
        ('value', 'Value'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_mapping(
            mapping_id=parsed_args.mapping_id)
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliGetMapping, self).get_parser(prog_name)
        parser.add_argument('mapping_id', type=str,
                            help='Mapping ID to filter on')
        return parser


class CliListMapping(lister.Lister):
    """List hashmap mappings."""

    columns = [
        ('mapping_id', 'Mapping ID'),
        ('value', 'Value'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_mapping(
            **vars(parsed_args))
        values = utils.list_to_cols(resp['mappings'], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliListMapping, self).get_parser(prog_name)
        parser.add_argument('-s', '--service-id', type=str,
                            help='Service ID to filter on')
        parser.add_argument('-g', '--group-id', type=str,
                            help='Group ID to filter on')
        parser.add_argument('--field-id', type=str,
                            help='Field ID to filter on')
        parser.add_argument('-p', '--project-id', type=str, dest='tenant_id',
                            help='Project ID to filter on')
        parser.add_argument('--filter-tenant', action='store_true',
                            help='Explicitly filter on given tenant (allows '
                            'to filter on tenant being None)')
        parser.add_argument('--no-group', action='store_true',
                            help='Filter on orphaned mappings')
        return parser


class CliCreateMapping(lister.Lister):
    """Create a Hashmap mapping."""
    columns = [
        ('mapping_id', 'Mapping ID'),
        ('value', 'Value'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.create_mapping(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCreateMapping, self).get_parser(prog_name)
        parser.add_argument('-s', '--service-id', type=str, help='Service ID')
        parser.add_argument('-g', '--group-id', type=str, help='Group ID')
        parser.add_argument('--field-id', type=str, help='Field ID')
        parser.add_argument('-p', '--project-id', type=str, dest='tenant_id',
                            help='Project ID')
        parser.add_argument('-t', '--type', type=str, help='Mapping type')
        parser.add_argument('--value', type=str, help='Value')
        parser.add_argument('cost', type=float, help='Cost')
        return parser


class CliDeleteMapping(command.Command):
    """Delete a Hashmap mapping."""

    def take_action(self, parsed_args):
        utils.get_client_from_osc(self).rating.hashmap.delete_mapping(
            **vars(parsed_args))

    def get_parser(self, prog_name):
        parser = super(CliDeleteMapping, self).get_parser(prog_name)
        parser.add_argument('mapping_id', type=str, help='Mapping ID')
        return parser


class CliUpdateMapping(lister.Lister):
    """Update a Hashmap mapping."""

    columns = [
        ('mapping_id', 'Mapping ID'),
        ('value', 'Value'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.update_mapping(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliUpdateMapping, self).get_parser(prog_name)
        parser.add_argument('-s', '--service-id', type=str, help='Service ID')
        parser.add_argument('-g', '--group-id', type=str, help='Group ID')
        parser.add_argument('--field-id', type=str, help='Field ID')
        parser.add_argument('-p', '--project-id', type=str, dest='tenant_id',
                            help='Project ID')
        parser.add_argument('--value', type=str, help='Value')
        parser.add_argument('--cost', type=str, help='Cost')
        parser.add_argument('mapping_id', type=str, help='Mapping ID')
        return parser


class CliListGroup(lister.Lister):
    """List existing hashmap groups."""

    columns = [
        ('name', 'Name'),
        ('group_id', 'Group ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_group()
        values = utils.list_to_cols(resp['groups'], self.columns)
        return [col[1] for col in self.columns], values


class CliCreateGroup(lister.Lister):
    """Create a Hashmap group."""
    columns = [
        ('name', 'Name'),
        ('group_id', 'Group ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.create_group(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCreateGroup, self).get_parser(prog_name)
        parser.add_argument('name', type=str, help='Group Name')
        return parser


class CliDeleteGroup(command.Command):
    """Create a Hashmap group."""

    def take_action(self, parsed_args):
        utils.get_client_from_osc(self).rating.hashmap.delete_group(
            **vars(parsed_args))

    def get_parser(self, prog_name):
        parser = super(CliDeleteGroup, self).get_parser(prog_name)
        parser.add_argument('--recursive', action='store_true',
                            help='Delete mappings recursively')
        parser.add_argument('group_id', type=str, help='Group ID')
        return parser


class CliGetGroupMappings(lister.Lister):
    """Get all Hashmap mappings for the given group."""

    columns = [
        ('mapping_id', 'Mapping ID'),
        ('value', 'Value'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        client = utils.get_client_from_osc(self)
        resp = client.rating.hashmap.get_group_mappings(**vars(parsed_args))
        return ([col[1] for col in self.columns],
                utils.list_to_cols(resp.get('mappings', []), self.columns))

    def get_parser(self, prog_name):
        parser = super(CliGetGroupMappings, self).get_parser(prog_name)
        parser.add_argument('group_id', type=str, help='Group ID')
        return parser


class CliGetGroupThresholds(lister.Lister):
    """Get all thresholds for the given group."""

    columns = [
        ('threshold_id', 'Threshold ID'),
        ('level', 'Level'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        client = utils.get_client_from_osc(self)
        resp = client.rating.hashmap.get_group_thresholds(**vars(parsed_args))
        return ([col[1] for col in self.columns],
                utils.list_to_cols(resp.get('thresholds', []), self.columns))

    def get_parser(self, prog_name):
        parser = super(CliGetGroupThresholds, self).get_parser(prog_name)
        parser.add_argument('group_id', type=str, help='Group ID')
        return parser


class CliGetThreshold(lister.Lister):
    """Get a Hashmap threshold."""

    columns = [
        ('threshold_id', 'Threshold ID'),
        ('level', 'Level'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_threshold(
            threshold_id=parsed_args.threshold_id,
        )
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliGetThreshold, self).get_parser(prog_name)
        parser.add_argument('threshold_id', type=str,
                            help='Threshold ID to filter on')
        return parser


class CliListThreshold(lister.Lister):
    """List Hashmap thresholds"""
    columns = [
        ('threshold_id', 'Threshold ID'),
        ('level', 'Level'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.get_threshold(
            **vars(parsed_args))
        values = utils.list_to_cols(resp['thresholds'], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliListThreshold, self).get_parser(prog_name)
        parser.add_argument('-s', '--service-id', type=str,
                            help='Service ID to filter on')
        parser.add_argument('-g', '--group-id', type=str,
                            help='Group ID to filter on')
        parser.add_argument('--field-id', type=str,
                            help='Field ID to filter on')
        parser.add_argument('-p', '--project-id', type=str, dest='tenant_id',
                            help='Project ID to filter on')
        parser.add_argument('--filter-tenant', action='store_true',
                            help='Explicitly filter on given tenant (allows '
                            'to filter on tenant being None)')
        parser.add_argument('--no-group', action='store_true',
                            help='Filter on orphaned thresholds')
        return parser


class CliCreateThreshold(lister.Lister):
    """Create a Hashmap threshold."""
    columns = [
        ('threshold_id', 'Threshold ID'),
        ('level', 'Level'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.create_threshold(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliCreateThreshold, self).get_parser(prog_name)
        parser.add_argument('-s', '--service-id', type=str, help='Service ID')
        parser.add_argument('-g', '--group-id', type=str, help='Group ID')
        parser.add_argument('--field-id', type=str, help='Field ID')
        parser.add_argument('-p', '--project-id', type=str, dest='tenant_id',
                            help='Project ID')
        parser.add_argument('-t', '--type', type=str, help='Threshold type')
        parser.add_argument('level', type=str, help='Threshold level')
        parser.add_argument('cost', type=float, help='Cost')
        return parser


class CliDeleteThreshold(command.Command):
    """Delete a Hashmap threshold."""

    def take_action(self, parsed_args):
        utils.get_client_from_osc(self).rating.hashmap.delete_threshold(
            **vars(parsed_args))

    def get_parser(self, prog_name):
        parser = super(CliDeleteThreshold, self).get_parser(prog_name)
        parser.add_argument('threshold_id', type=str, help='Threshold ID')
        return parser


class CliUpdateThreshold(lister.Lister):
    """Update a Hashmap threshold."""

    columns = [
        ('threshold_id', 'Threshold ID'),
        ('level', 'Level'),
        ('cost', 'Cost'),
        ('type', 'Type'),
        ('field_id', 'Field ID'),
        ('service_id', 'Service ID'),
        ('group_id', 'Group ID'),
        ('tenant_id', 'Project ID'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).rating.hashmap.update_threshold(
            **vars(parsed_args))
        values = utils.list_to_cols([resp], self.columns)
        return [col[1] for col in self.columns], values

    def get_parser(self, prog_name):
        parser = super(CliUpdateThreshold, self).get_parser(prog_name)
        parser.add_argument('-s', '--service-id', type=str, help='Service ID')
        parser.add_argument('-g', '--group-id', type=str, help='Group ID')
        parser.add_argument('--field-id', type=str, help='Field ID')
        parser.add_argument('-p', '--project-id', type=str, dest='tenant_id',
                            help='Project ID')
        parser.add_argument('-l', '--level', type=str, help='Threshold level')
        parser.add_argument('--cost', type=str, help='Cost')
        parser.add_argument('threshold_id', type=str, help='Threshold ID')
        return parser



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import jsonpath_rw_ext as jp

from cloudkittyclient.tests.functional import base


class CkInfoTest(base.BaseFunctionalTest):

    def __init__(self, *args, **kwargs):
        super(CkInfoTest, self).__init__(*args, **kwargs)
        self.runner = self.cloudkitty

    def test_info_config_get(self):
        resp = self.runner('info config get')
        for elem in resp:
            if elem.get('Section') == 'name':
                self.assertEqual(elem['Value'], 'OpenStack')

    def test_info_metric_list(self):
        resp = self.runner('info metric list')
        res = jp.match1('$.[*].Metric', resp)
        self.assertIsNotNone(res)

    def test_info_service_get_image_size(self):
        resp = self.runner('info metric get', params='image.size')[0]
        self.assertEqual(resp['Metric'], 'image.size')


class OSCInfoTest(CkInfoTest):

    def __init__(self, *args, **kwargs):
        super(OSCInfoTest, self).__init__(*args, **kwargs)
        self.runner = self.openstack



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient import exc
from cloudkittyclient.tests.unit.v1 import base


class TestPyscripts(base.BaseAPIEndpointTestCase):

    def test_list_scripts(self):
        self.pyscripts.list_scripts()
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/pyscripts/scripts/')

    def test_list_scripts_no_data(self):
        self.pyscripts.list_scripts(no_data=True)
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/pyscripts/scripts/?no_data=True')

    def test_get_script(self):
        self.pyscripts.get_script(script_id='testscript')
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/pyscripts/scripts/testscript')

    def test_get_script_no_arg(self):
        self.assertRaises(exc.ArgumentRequired, self.pyscripts.get_script)

    def test_create_script(self):
        kwargs = dict(name='name', data='data')
        self.pyscripts.create_script(**kwargs)
        self.api_client.post.assert_called_once_with(
            '/v1/rating/module_config/pyscripts/scripts/', json=kwargs)

    def test_create_script_no_data(self):
        self.assertRaises(
            exc.ArgumentRequired, self.pyscripts.create_script, name='name')

    def test_create_script_no_name(self):
        self.assertRaises(
            exc.ArgumentRequired, self.pyscripts.create_script, data='data')

    def test_update_script(self):
        args = dict(script_id='script_id', name='name', data='data')
        self.pyscripts.update_script(**args)
        self.api_client.get.assert_called_once_with(
            '/v1/rating/module_config/pyscripts/scripts/script_id')
        args.pop('script_id', None)
        self.api_client.put.assert_called_once_with(
            '/v1/rating/module_config/pyscripts/scripts/script_id', json=args)

    def test_update_script_no_script_id(self):
        self.assertRaises(
            exc.ArgumentRequired, self.pyscripts.update_script, name='name')

    def test_delete_script(self):
        kwargs = dict(script_id='script_id')
        self.pyscripts.delete_script(**kwargs)
        self.api_client.delete.assert_called_once_with(
            '/v1/rating/module_config/pyscripts/scripts/script_id')



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from string import Formatter as StringFormatter
from urllib.parse import urlencode

from cloudkittyclient import utils


class HttpDecoratorMeta(type):

    ignore = ('get_url', )

    def __new__(cls, *args, **kwargs):
        return utils.format_http_errors(HttpDecoratorMeta.ignore)(
            super(HttpDecoratorMeta, cls).__new__(cls, *args, **kwargs)
        )


class BaseManager(object, metaclass=HttpDecoratorMeta):
    """Base class for Endpoint Manager objects."""

    url = ''

    def __init__(self, api_client):
        self.api_client = api_client
        self._formatter = StringFormatter()

    def _get_format_kwargs(self, **kwargs):
        it = self._formatter.parse(self.url)
        output = {i[1]: '' for i in it}
        for key in output.keys():
            if kwargs.get(key):
                output[key] = kwargs[key]
        if 'endpoint' in output.keys():
            output.pop('endpoint')
        return output

    def get_url(self,
                endpoint,
                kwargs,
                authorized_args=[]):
        """Returns the required url for a request against CloudKitty's API.

        :param endpoint: The endpoint on which the request should be done
        :type endpoint: str
        :param kwargs: kwargs that will be used to build the query (part after
                       '?' in the url) and to format the url.
        :type kwargs: dict
        :param authorized_args: The arguments that are authorized in url
                                parameters
        :type authorized_args: list
        """
        query_kwargs = {
            key: kwargs[key] for key in authorized_args
            if kwargs.get(key, None)
        }
        kwargs = self._get_format_kwargs(**kwargs)
        url = self.url.format(endpoint=endpoint, **kwargs)
        query = urlencode(query_kwargs)
        if query:
            url += '?' + query
        return url



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.common import base
from cloudkittyclient import exc


class ScopeManager(base.BaseManager):
    """Class used to handle /v2/scope endpoint"""

    url = '/v2/scope'

    def get_scope_state(self, **kwargs):
        """Returns a paginated list of scopes along with their state.

        Some optional filters can be provided.

        :param offset: Index of the first scope that should be returned.
        :type offset: int
        :param limit: Maximal number of scopes to return.
        :type limit: int
        :param collector: Optional collector to filter on.
        :type collector: str or list of str
        :param fetcher: Optional fetcher to filter on.
        :type fetcher: str or list of str
        :param scope_id: Optional scope_id to filter on.
        :type scope_id: str or list of str
        :param scope_key: Optional scope_key to filter on.
        :type scope_key: str or list of str
        """

        for key in ('collector', 'fetcher', 'scope_id', 'scope_key'):
            if key in kwargs.keys():
                if isinstance(kwargs[key], list):
                    kwargs[key] = ','.join(kwargs[key])

        authorized_args = [
            'offset', 'limit', 'collector', 'fetcher', 'scope_id', 'scope_key']
        url = self.get_url(None, kwargs, authorized_args=authorized_args)
        return self.api_client.get(url).json()

    def reset_scope_state(self, **kwargs):
        """Returns nothing.

        Some optional filters can be provided.
        The all_scopes and the scope_id options are mutually exclusive and one
        must be provided.

        :param state: datetime object from which the state will be reset
        :type state: datetime.datetime
        :param all_scopes: Whether all scopes must be reset
        :type all_scopes: bool
        :param collector: Optional collector to filter on.
        :type collector: str or list of str
        :param fetcher: Optional fetcher to filter on.
        :type fetcher: str or list of str
        :param scope_id: Optional scope_id to filter on.
        :type scope_id: str or list of str
        :param scope_key: Optional scope_key to filter on.
        :type scope_key: str or list of str
        """

        if not kwargs.get('state'):
            raise exc.ArgumentRequired("'state' argument is required")

        if not kwargs.get('all_scopes') and not kwargs.get('scope_id'):
            raise exc.ArgumentRequired(
                "You must specify either 'scope_id' or 'all_scopes'")

        if kwargs.get('all_scopes') and kwargs.get('scope_id'):
            raise exc.InvalidArgumentError(
                "You can't specify both 'scope_id' and 'all_scopes'")

        for key in ('collector', 'fetcher', 'scope_id', 'scope_key'):
            if key in kwargs.keys():
                if isinstance(kwargs[key], list):
                    kwargs[key] = ','.join(kwargs[key])

        body = dict(
            state=kwargs.get('state'),
            scope_id=kwargs.get('scope_id'),
            scope_key=kwargs.get('scope_key'),
            collector=kwargs.get('collector'),
            fetcher=kwargs.get('fetcher'),
            all_scopes=kwargs.get('all_scopes'),
        )
        # Stripping None and False values
        body = dict(filter(lambda elem: bool(elem[1]), body.items()))

        url = self.get_url(None, kwargs)
        return self.api_client.put(url, json=body)



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import lister
from oslo_utils import timeutils

from cloudkittyclient import utils


class CliSummaryGet(lister.Lister):
    """Get a summary for a given period."""

    def get_parser(self, prog_name):
        parser = super(CliSummaryGet, self).get_parser(prog_name)

        def filter_(elem):
            if len(elem.split(':')) != 2:
                raise TypeError
            return str(elem)

        parser.add_argument('--offset', type=int, default=0,
                            help='Index of the first element')
        parser.add_argument('--limit', type=int, default=100,
                            help='Maximal number of elements')
        parser.add_argument('-g', '--groupby', type=str, action='append',
                            help='Attribute to group the summary by. Can be '
                            'specified several times')
        parser.add_argument('--filter', type=filter_, action='append',
                            help="Optional filter, in 'key:value' format. Can "
                            "be specified several times.")
        parser.add_argument('-b', '--begin', type=timeutils.parse_isotime,
                            help="Start of the period to query, in iso8601 "
                            "format. Example: 2019-05-01T00:00:00Z.")
        parser.add_argument('-e', '--end', type=timeutils.parse_isotime,
                            help="End of the period to query, in iso8601 "
                            "format. Example: 2019-06-01T00:00:00Z.")

        return parser

    def take_action(self, parsed_args):
        filters = dict(elem.split(':') for elem in (parsed_args.filter or []))
        resp = utils.get_client_from_osc(self).summary.get_summary(
            offset=parsed_args.offset,
            limit=parsed_args.limit,
            begin=parsed_args.begin,
            end=parsed_args.end,
            filters=filters,
            groupby=parsed_args.groupby,
        )
        columns = [c.replace('_', ' ').capitalize() for c in resp['columns']]
        return columns, resp['results']



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.unit.v1 import base


class TestReport(base.BaseAPIEndpointTestCase):

    def test_get_summary(self):
        self.report.get_summary()
        self.api_client.get.assert_called_once_with('/v1/report/summary')

    def test_get_summary_with_groupby(self):
        self.report.get_summary(groupby=['res_type', 'tenant_id'])
        self.api_client.get.assert_called_once_with(
            '/v1/report/summary?groupby=res_type%2Ctenant_id')

    def test_get_summary_with_begin_end(self):
        self.report.get_summary(begin='begin', end='end')
        try:
            self.api_client.get.assert_called_once_with(
                '/v1/report/summary?begin=begin&end=end')
        # Passing a dict to urlencode can change arg order
        except AssertionError:
            self.api_client.get.assert_called_once_with(
                '/v1/report/summary?end=end&begin=begin')

    def test_get_total(self):
        self.report.get_total()
        self.api_client.get.assert_called_once_with('/v1/report/total')

    def test_get_total_with_begin_end(self):
        self.report.get_total(begin='begin', end='end')
        try:
            self.api_client.get.assert_called_once_with(
                '/v1/report/total?begin=begin&end=end')
        # Passing a dict to urlencode can change arg order
        except AssertionError:
            self.api_client.get.assert_called_once_with(
                '/v1/report/total?end=end&begin=begin')

    def test_get_tenants(self):
        self.report.get_tenants()
        self.api_client.get.assert_called_once_with('/v1/report/tenants')

    def test_get_tenants_with_begin_end(self):
        self.report.get_tenants(begin='begin', end='end')
        try:
            self.api_client.get.assert_called_once_with(
                '/v1/report/tenants?begin=begin&end=end')
        # Passing a dict to urlencode can change arg order
        except AssertionError:
            self.api_client.get.assert_called_once_with(
                '/v1/report/tenants?end=end&begin=begin')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import sys


class BaseException(Exception):
    """An error occurred."""
    def __init__(self, message=None):
        self.message = message

    def __str__(self):
        return self.message or self.__class__.__doc__


class CommandError(BaseException):
    """Invalid usage of CLI."""


class InvalidEndpoint(BaseException):
    """The provided endpoint is invalid."""


class ArgumentRequired(BaseException):
    """A required argument was not provided."""


class CommunicationError(BaseException):
    """Unable to communicate with server."""


class InvalidArgumentError(BaseException):
    """Exception raised when a provided argument is invalid"""


class NotUpdatableError(BaseException):
    """This Resource is not updatable."""

    def __init__(self, resource):
        message = "%s is not updatable" % resource
        super(BaseException, self).__init__(message)


class HTTPException(BaseException):
    """Base exception for all HTTP-derived exceptions."""
    code = 'N/A'

    def __init__(self, details=None):
        self.details = details

    def __str__(self):
        try:
            data = json.loads(self.details)
            message = data.get("error_message", {}).get("faultstring")
            if message:
                return "%s (HTTP %s) ERROR %s" % (
                    self.__class__.__name__, self.code, message)
        except (ValueError, TypeError, AttributeError):
            pass
        return "%s (HTTP %s)" % (self.__class__.__name__, self.code)


class HTTPBadRequest(HTTPException):
    code = 400


class HTTPUnauthorized(HTTPException):
    code = 401


class HTTPForbidden(HTTPException):
    code = 403


class HTTPNotFound(HTTPException):
    code = 404


class HTTPMethodNotAllowed(HTTPException):
    code = 405


class HTTPConflict(HTTPException):
    code = 409


class HTTPOverLimit(HTTPException):
    code = 413


class HTTPInternalServerError(HTTPException):
    code = 500


class HTTPNotImplemented(HTTPException):
    code = 501


class HTTPBadGateway(HTTPException):
    code = 502


class HTTPServiceUnavailable(HTTPException):
    code = 503


# NOTE(bcwaldon): Build a mapping of HTTP codes to corresponding exception
# classes
_code_map = {}
for obj_name in dir(sys.modules[__name__]):
    if obj_name.startswith('HTTP'):
        obj = getattr(sys.modules[__name__], obj_name)
        _code_map[obj.code] = obj


def from_response(response, details=None):
    """Return an instance of an HTTPException based on httplib response."""
    cls = _code_map.get(response.status, HTTPException)
    return cls(details)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.functional import base


class CkReportTest(base.BaseFunctionalTest):
    def __init__(self, *args, **kwargs):
        super(CkReportTest, self).__init__(*args, **kwargs)
        self.runner = self.cloudkitty

    def test_get_summary(self):
        resp = self.runner('summary get')
        self.assertEqual(len(resp), 0)

    def test_get_summary_with_groupby(self):
        resp = self.runner('summary get', params='-g res_type tenant_id')
        self.assertEqual(len(resp), 0)

    def test_get_total(self):
        resp = self.runner('total get')
        self.assertIn('Total', resp.keys())

    def test_get_tenants(self):
        self.runner('report tenant list')


class OSCReportTest(CkReportTest):

    def __init__(self, *args, **kwargs):
        super(OSCReportTest, self).__init__(*args, **kwargs)
        self.runner = self.openstack



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from keystoneauth1 import adapter
from keystoneauth1 import session as ks_session


class BaseClient(object):

    def __init__(self,
                 session=None,
                 adapter_options={},
                 cacert=None,
                 insecure=False,
                 **kwargs):
        adapter_options.setdefault('service_type', 'rating')
        adapter_options.setdefault('additional_headers', {
            'Content-Type': 'application/json',
        })

        if insecure:
            verify_cert = False
        else:
            if cacert:
                verify_cert = cacert
            else:
                verify_cert = True

        self.session = session
        if self.session is None:
            self.session = ks_session.Session(
                verify=verify_cert, **kwargs)

        self.api_client = adapter.Adapter(
            session=self.session, **adapter_options)



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import argparse

from cliff import command
from cliff import lister
from oslo_utils import timeutils

from cloudkittyclient import utils


class CliDataframesAdd(command.Command):
    """Add one or several DataFrame objects to the storage backend."""
    def get_parser(self, prog_name):
        parser = super(CliDataframesAdd, self).get_parser(prog_name)

        parser.add_argument(
            'datafile',
            type=argparse.FileType('r'),
            help="File formatted as a JSON object having a DataFrame list"
                 "under a 'dataframes' key."
                 "'-' (hyphen) can be specified for using stdin.",
            )

        return parser

    def take_action(self, parsed_args):
        with parsed_args.datafile as dfile:
            dataframes = dfile.read()
            utils.get_client_from_osc(self).dataframes.add_dataframes(
                dataframes=dataframes,
            )


class CliDataframesGet(lister.Lister):
    """Get dataframes from the storage backend."""
    columns = [
        ('begin', 'Begin'),
        ('end', 'End'),
        ('metric', 'Metric Type'),
        ('unit', 'Unit'),
        ('qty', 'Quantity'),
        ('price', 'Price'),
        ('groupby', 'Group By'),
        ('metadata', 'Metadata'),
    ]

    def get_parser(self, prog_name):
        parser = super(CliDataframesGet, self).get_parser(prog_name)

        def filter_(elem):
            if len(elem.split(':')) != 2:
                raise TypeError
            return str(elem)

        parser.add_argument('--offset', type=int, default=0,
                            help='Index of the first dataframe')
        parser.add_argument('--limit', type=int, default=100,
                            help='Maximal number of dataframes')
        parser.add_argument('--filter', type=filter_, action='append',
                            help="Optional filter, in 'key:value' format. Can "
                            "be specified several times.")
        parser.add_argument('-b', '--begin', type=timeutils.parse_isotime,
                            help="Start of the period to query, in iso8601 "
                            "format. Example: 2019-05-01T00:00:00Z.")
        parser.add_argument('-e', '--end', type=timeutils.parse_isotime,
                            help="End of the period to query, in iso8601 "
                            "format. Example: 2019-06-01T00:00:00Z.")

        return parser

    def take_action(self, parsed_args):
        filters = dict(elem.split(':') for elem in (parsed_args.filter or []))

        dataframes = utils.get_client_from_osc(self).dataframes.get_dataframes(
            offset=parsed_args.offset,
            limit=parsed_args.limit,
            begin=parsed_args.begin,
            end=parsed_args.end,
            filters=filters,
        ).get('dataframes', [])

        def format_(d):
            return ' '.join([
                '{}="{}"'.format(k, v) for k, v in (d or {}).items()])

        values = []
        for df in dataframes:
            period = df['period']
            usage = df['usage']
            for metric_type, points in usage.items():
                for point in points:
                    values.append([
                        period['begin'],
                        period['end'],
                        metric_type,
                        point['vol']['unit'],
                        point['vol']['qty'],
                        point['rating']['price'],
                        format_(point.get('groupby', {})),
                        format_(point.get('metadata', {})),
                    ])

        return [col[1] for col in self.columns], values



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.common import base
from cloudkittyclient import exc


class PyscriptManager(base.BaseManager):
    """Class used to manage the Pyscript rating module"""

    url = '/v1/rating/module_config/pyscripts/{endpoint}/{script_id}'

    def list_scripts(self, **kwargs):
        """Get a list of all pyscripts.

        :param no_data: Set to True to remove script data from output.
        :type no_data: bool
        """
        authorized_args = ['no_data']
        url = self.get_url('scripts', kwargs, authorized_args)
        return self.api_client.get(url).json()

    def get_script(self, **kwargs):
        """Get the script corresponding to the given ID.

        :param script_id: ID of the script.
        :type script_id: str
        """
        if not kwargs.get('script_id'):
            raise exc.ArgumentRequired("Argument 'script_id' is required.")
        url = self.get_url('scripts', kwargs)
        return self.api_client.get(url).json()

    def create_script(self, **kwargs):
        """Create a new script.

        :param name: Name of the script to create
        :type name: str
        :param data: Content of the script
        :type data: str
        """
        for arg in ('name', 'data'):
            if not kwargs.get(arg):
                raise exc.ArgumentRequired(
                    "'Argument {} is required.'".format(arg))
        url = self.get_url('scripts', kwargs)
        body = dict(name=kwargs['name'], data=kwargs['data'])
        return self.api_client.post(url, json=body).json()

    def update_script(self, **kwargs):
        """Update an existing script.

        :param script_id: ID of the script to update
        :type script_id: str
        :param name: Name of the script to create
        :type name: str
        :param data: Content of the script
        :type data: str
        """
        if not kwargs.get('script_id'):
            raise exc.ArgumentRequired("Argument 'script_id' is required.")
        script = self.get_script(script_id=kwargs['script_id'])
        for key in ('name', 'data'):
            if kwargs.get(key):
                script[key] = kwargs[key]
        script.pop('checksum', None)
        url = self.get_url('scripts', kwargs)
        return self.api_client.put(url, json=script).json()

    def delete_script(self, **kwargs):
        """Delete a script.

        :param script_id: ID of the script to update
        :type script_id: str
        """
        if not kwargs.get('script_id'):
            raise exc.ArgumentRequired("Argument 'script_id' is required.")
        url = self.get_url('scripts', kwargs)
        self.api_client.delete(url)



# Copyright 2019 objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from cloudkittyclient.tests import utils
from cloudkittyclient.v2 import dataframes
from cloudkittyclient.v2.rating import modules
from cloudkittyclient.v2 import scope
from cloudkittyclient.v2 import summary


class BaseAPIEndpointTestCase(utils.BaseTestCase):

    def setUp(self):
        super(BaseAPIEndpointTestCase, self).setUp()
        self.api_client = utils.FakeHTTPClient()
        self.dataframes = dataframes.DataframesManager(self.api_client)
        self.scope = scope.ScopeManager(self.api_client)
        self.summary = summary.SummaryManager(self.api_client)
        self.rating = modules.RatingManager(self.api_client)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.functional import base


class CkCollectorTest(base.BaseFunctionalTest):

    def __init__(self, *args, **kwargs):
        super(CkCollectorTest, self).__init__(*args, **kwargs)
        self.runner = self.cloudkitty

    def test_create_get_delete_collector_mapping(self):
        # Create Mapping
        resp = self.runner(
            'collector-mapping create', params='compute gnocchi')[0]
        self.assertEqual(resp['Collector'], 'gnocchi')
        self.assertEqual(resp['Service'], 'compute')

        # Check that mapping is queryable
        resp = self.runner('collector-mapping list')
        self.assertEqual(len(resp), 1)
        resp = resp[0]
        self.assertEqual(resp['Collector'], 'gnocchi')
        self.assertEqual(resp['Service'], 'compute')

        # Delete mapping
        self.runner('collector-mapping delete',
                    params='compute', has_output=False)

        # Check that mapping was deleted
        resp = self.runner('collector-mapping list')
        self.assertEqual(len(resp), 0)

    def test_collector_enable_disable(self):
        # Enable collector
        resp = self.runner('collector enable gnocchi')
        self.assertEqual(len(resp), 1)
        resp = resp[0]
        self.assertEqual(resp['Collector'], 'gnocchi')
        self.assertEqual(resp['State'], True)

        # Disable collector
        resp = self.runner('collector disable gnocchi')
        self.assertEqual(len(resp), 1)
        resp = resp[0]
        self.assertEqual(resp['Collector'], 'gnocchi')
        self.assertEqual(resp['State'], False)


class OSCCollectorTest(CkCollectorTest):

    def __init__(self, *args, **kwargs):
        super(OSCCollectorTest, self).__init__(*args, **kwargs)
        self.runner = self.openstack



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import command
from cliff import lister
from oslo_utils import timeutils

from cloudkittyclient import utils


class CliScopeStateGet(lister.Lister):
    """Get information about current state of several scopes."""
    info_columns = [
        ('scope_id', 'Scope ID'),
        ('scope_key', 'Scope Key'),
        ('collector', 'Collector'),
        ('fetcher', 'Fetcher'),
        ('state', 'State')
    ]

    def get_parser(self, prog_name):
        parser = super(CliScopeStateGet, self).get_parser(prog_name)

        for col in self.info_columns[:-1]:
            parser.add_argument(
                '--' + col[0].replace('_', '-'), type=str,
                action='append', help='Optional filter on ' + col[1])

        parser.add_argument('--offset', type=int, default=0,
                            help='Index of the first scope')
        parser.add_argument('--limit', type=int, default=100,
                            help='Maximal number of scopes')

        return parser

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).scope.get_scope_state(
            offset=parsed_args.offset,
            limit=parsed_args.limit,
            collector=parsed_args.collector,
            fetcher=parsed_args.fetcher,
            scope_id=parsed_args.scope_id,
            scope_key=parsed_args.scope_key,
        )
        values = utils.list_to_cols(resp['results'], self.info_columns)
        return [col[1] for col in self.info_columns], values


class CliScopeStateReset(command.Command):
    """Reset the state of several scopes."""
    info_columns = [
        ('scope_id', 'Scope ID'),
        ('scope_key', 'Scope Key'),
        ('collector', 'Collector'),
        ('fetcher', 'Fetcher'),
    ]

    def get_parser(self, prog_name):
        parser = super(CliScopeStateReset, self).get_parser(prog_name)

        for col in self.info_columns:
            parser.add_argument(
                '--' + col[0].replace('_', '-'), type=str,
                action='append', help='Optional filter on ' + col[1])

        parser.add_argument(
            '-a', '--all-scopes',
            action='store_true',
            help="Target all scopes at once")

        parser.add_argument(
            'state',
            type=timeutils.parse_isotime,
            help="State iso8601 datetime to which the state should be set. "
            "Example: 2019-06-01T00:00:00Z.")

        return parser

    def take_action(self, parsed_args):
        utils.get_client_from_osc(self).scope.reset_scope_state(
            collector=parsed_args.collector,
            fetcher=parsed_args.fetcher,
            scope_id=parsed_args.scope_id,
            scope_key=parsed_args.scope_key,
            all_scopes=parsed_args.all_scopes,
            state=parsed_args.state,
        )



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.functional import base


class CkRatingTest(base.BaseFunctionalTest):

    def __init__(self, *args, **kwargs):
        super(CkRatingTest, self).__init__(*args, **kwargs)
        self.runner = self.cloudkitty

    def test_module_enable_get_disable(self):
        # enable
        resp = self.runner('module enable', params='hashmap')[0]
        self.assertTrue(resp['Enabled'])

        # get
        resp = self.runner('module get', params='hashmap')[0]
        self.assertTrue(resp['Enabled'])
        self.assertEqual(resp['Module'], 'hashmap')

        # disable
        resp = self.runner('module disable', params='hashmap')[0]
        self.assertFalse(resp['Enabled'])

    def test_module_set_priority(self):
        resp = self.runner('module set priority', params='hashmap 100')[0]
        self.assertEqual(resp['Priority'], 100)


class OSCRatingTest(CkRatingTest):

    def __init__(self, *args, **kwargs):
        super(CkRatingTest, self).__init__(*args, **kwargs)
        self.runner = self.openstack



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import decimal

from cloudkittyclient import exc
from cloudkittyclient.tests.unit.v1 import base


class TestRating(base.BaseAPIEndpointTestCase):

    def test_quote_request(self):
        res_data = [{'usage': {
            'instance': [{
                'vol': {'unit': 'undef', 'qty': '1'},
                'rating': {'price': decimal.Decimal(1)},
                'desc': {
                    'disk_total_display': 1,
                    'image_id': 'c43a3e7d-c4e6-45d6-8c8d-e2832a45bc0a',
                    'ram': 64,
                    'ephemeral': 0,
                    'vcpus': 1,
                    'source_type': 'image',
                    'disk_total': 1,
                    'flavor_id': '42',
                    'flavor': 'm1.nano',
                    'disk': 1,
                    'source_val': 'c43a3e7d-c4e6-45d6-8c8d-e2832a45bc0a'}
            }]
        }}]

        self.rating.get_quotation(res_data=res_data)
        self.api_client.post.assert_called_once_with(
            '/v1/rating/quote/', json={'resources': res_data})

    def test_get_quotation_no_res_data(self):
        self.assertRaises(exc.ArgumentRequired, self.rating.get_quotation)



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import lister

from cloudkittyclient import utils


class CliInfoMetricGet(lister.Lister):
    """Get information about current metrics."""
    info_columns = [
        ('metric_id', 'Metric'),
        ('unit', 'Unit'),
        ('metadata', 'Metadata'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).info.get_metric(
            metric_name=parsed_args.metric_name,
        )
        values = utils.list_to_cols([resp], self.info_columns)
        return [col[1] for col in self.info_columns], values

    def get_parser(self, prog_name):
        parser = super(CliInfoMetricGet, self).get_parser(prog_name)
        parser.add_argument('metric_name',
                            type=str, default='', help='Metric name')
        return parser


class CliInfoMetricList(lister.Lister):
    """Get information about a single metric."""
    info_columns = [
        ('metric_id', 'Metric'),
        ('unit', 'Unit'),
        ('metadata', 'Metadata'),
    ]

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).info.get_metric()
        values = utils.list_to_cols(resp['metrics'], self.info_columns)
        return [col[1] for col in self.info_columns], values


class CliInfoConfigGet(lister.Lister):
    """Get information about the current configuration."""
    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).info.get_config()
        values = [(key, value) for key, value in resp.items()]
        return ('Section', 'Value'), values



# -*- coding: utf-8 -*-
# Copyright 2018 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cloudkittyclient.tests.unit.v1 import base


class TestStorage(base.BaseAPIEndpointTestCase):

    def test_get_dataframes(self):
        self.storage.get_dataframes()
        self.api_client.get.assert_called_once_with('/v1/storage/dataframes')

    def test_get_dataframes_with_begin_end(self):
        self.storage.get_dataframes(begin='begin', end='end')
        try:
            self.api_client.get.assert_called_once_with(
                '/v1/storage/dataframes?begin=begin&end=end')
        # Passing a dict to urlencode can change arg order
        except AssertionError:
            self.api_client.get.assert_called_once_with(
                '/v1/storage/dataframes?end=end&begin=begin')



# Copyright 2019 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from cliff import lister

from cloudkittyclient import utils


class CliModuleList(lister.Lister):
    """Get loaded rating modules list"""

    def get_parser(self, prog_name):
        parser = super(CliModuleList, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        resp = utils.get_client_from_osc(self).ratingmodules.get_modules_list()
        return resp['modules']
